@@ -1,15 +1,16 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -56,6 +57,7 @@
 #include "xcom/xcom_detector.h"
 #include "xcom/xcom_memory.h"
 #include "xcom/xcom_msg_queue.h"
+#include "xcom/xcom_scope_guard.h"
 #include "xcom/xcom_statistics.h"
 #include "xcom/xcom_transport.h"
 #include "xcom/xcom_vp_str.h"
@@ -102,11 +104,12 @@ int close_open_connection(connection_descriptor *conn) {
   return Network_provider_manager::getInstance().close_xcom_connection(conn);
 }
 
-connection_descriptor *open_new_connection(const char *server, xcom_port port,
-                                           int connection_timeout) {
+connection_descriptor *open_new_connection(
+    const char *server, xcom_port port, int connection_timeout,
+    network_provider_dynamic_log_level log_level) {
   return Network_provider_manager::getInstance().open_xcom_connection(
       server, port, Network_provider_manager::getInstance().is_xcom_using_ssl(),
-      connection_timeout);
+      connection_timeout, log_level);
 }
 
 /* purecov: begin deadcode */
@@ -129,6 +132,18 @@ connection_descriptor *open_new_local_connection(const char *server,
 }
 /* purecov: end */
 
+bool is_able_to_connect_to_node(const char *server, const xcom_port port) {
+  connection_descriptor *new_conn{nullptr};
+
+  Xcom_scope_guard cleanup_guard([&]() { free(new_conn); });
+
+  if (new_conn = open_new_connection(server, port, 1000); new_conn->fd == -1) {
+    return false;
+  }
+
+  return !static_cast<bool>(close_open_connection(new_conn));
+}
+
 result set_nodelay(int fd) {
   int n = 1;
   result ret = {0, 0};
@@ -417,6 +432,7 @@ int send_proto(connection_descriptor *con, xcom_proto x_proto,
 #define XDRFUNC xdrfunc
 #endif
 
+// This function is annotated with SUPPRESS_UBSAN, see header file declaration.
 int apply_xdr(void *buff, uint32_t bufflen, xdrproc_t xdrfunc, void *xdrdata,
               enum xdr_op op) {
   XDR xdr;
@@ -788,7 +804,8 @@ int incoming_connection_task(task_arg arg [[maybe_unused]]) {
 void server_detected(server *s) { s->detected = task_now(); }
 
 /* Try to connect to another node */
-static int dial(server *s) {
+static int dial(server *s,
+                network_provider_dynamic_log_level dial_call_log_level) {
   DECL_ENV
   int dummy;
   ENV_INIT
@@ -804,7 +821,7 @@ static int dial(server *s) {
   X_FREE(s->con);
   s->con = nullptr;
 
-  s->con = open_new_connection(s->srv, s->port, 1000);
+  s->con = open_new_connection(s->srv, s->port, 1000, dial_call_log_level);
   if (!s->con) {
     s->con = new_connection(-1, nullptr);
   }
@@ -1410,8 +1427,36 @@ linkage connect_wait = {
 
 void wakeup_sender() { task_wakeup(&connect_wait); }
 
+static server *find_server(server *table[], int n, char *name, xcom_port port) {
+  int i;
+  for (i = 0; i < n; i++) {
+    server *s = table[i];
+    if (s && strcmp(s->srv, name) == 0 &&
+        s->port == port) /* FIXME should use IP address */
+      return s;
+  }
+  return nullptr;
+}
+
 #define TAG_START 313
 
+/**
+ * @brief Checks if a given server is currently in use in the current view
+ *
+ * @param sp the server to check
+ *
+ * @return true if it is in the current view, false otherwise.
+ */
+bool is_server_in_current_view(server *sp) {
+  if (site_def *current_site_def = get_site_def_rw(); current_site_def) {
+    return find_server(current_site_def->servers,
+                       current_site_def->nodes.node_list_len, sp->srv,
+                       sp->port);
+  }
+
+  return false;
+}
+
 /* Fetch messages from queue and send to other server.  Having a
    separate queue and task for doing this simplifies the logic since we
    never need to wait to send. */
@@ -1425,6 +1470,7 @@ int sender_task(task_arg arg) {
 #if defined(_WIN32)
   bool was_connected;
 #endif
+  network_provider_dynamic_log_level dial_call_log_level;
   ENV_INIT
   END_ENV_INIT
   END_ENV;
@@ -1432,6 +1478,8 @@ int sender_task(task_arg arg) {
   int64_t ret_code{0};
   TASK_BEGIN
 
+  ep->dial_call_log_level = network_provider_dynamic_log_level::PROVIDED;
+
   ep->channel_empty_time = task_now();
   ep->dtime = INITIAL_CONNECT_WAIT; /* Initial wait is short, to avoid
                                        unnecessary waiting */
@@ -1450,7 +1498,7 @@ int sender_task(task_arg arg) {
 #if defined(_WIN32)
       if (!ep->was_connected) {
 #endif
-        TASK_CALL(dial(ep->s));
+        TASK_CALL(dial(ep->s, ep->dial_call_log_level));
 #if defined(_WIN32)
       } else {
         ep->s->reconnect = true;
@@ -1458,8 +1506,17 @@ int sender_task(task_arg arg) {
 #endif
       if (is_connected(ep->s->con)) break;
 
+      // Check, for each reconnect loop, if we belong to this configuration
+      ep->dial_call_log_level =
+          is_server_in_current_view(ep->s)
+              ? network_provider_dynamic_log_level::PROVIDED
+              : network_provider_dynamic_log_level::DEBUG;
+
       if (ep->dtime < MAX_CONNECT_WAIT) {
-        G_MESSAGE("Connection to %s:%d failed", ep->s->srv, ep->s->port);
+        if (ep->dial_call_log_level !=
+            network_provider_dynamic_log_level::DEBUG) {
+          G_MESSAGE("Connection to %s:%d failed", ep->s->srv, ep->s->port);
+        }
       }
 
       TIMED_TASK_WAIT(&connect_wait, ep->dtime);
@@ -1591,6 +1648,7 @@ int tcp_reconnection_task(task_arg arg [[maybe_unused]]) {
   int dummy;
   server *s;
   int i;
+  network_provider_dynamic_log_level dial_call_log_level;
   ENV_INIT
   END_ENV_INIT
   END_ENV;
@@ -1598,14 +1656,22 @@ int tcp_reconnection_task(task_arg arg [[maybe_unused]]) {
 
   ep->s = nullptr;
   ep->i = 0;
+  ep->dial_call_log_level = network_provider_dynamic_log_level::PROVIDED;
 
   while (!xcom_shutdown) {
     {
       ep->s = nullptr;
       for (ep->i = 0; ep->i < maxservers; ep->i++) {
         ep->s = all_servers[ep->i];
+
         if (ep->s && ep->s->reconnect && !is_connected(ep->s->con)) {
-          TASK_CALL(dial(ep->s));
+          // Check if the server belongs in the current view
+          ep->dial_call_log_level =
+              is_server_in_current_view(ep->s)
+                  ? network_provider_dynamic_log_level::PROVIDED
+                  : network_provider_dynamic_log_level::DEBUG;
+
+          TASK_CALL(dial(ep->s, ep->dial_call_log_level));
           if (is_connected(ep->s->con)) {
             ep->s->reconnect = false;
           }
@@ -1664,17 +1730,6 @@ int local_sender_task(task_arg arg) {
   TASK_END;
 }
 
-static server *find_server(server *table[], int n, char *name, xcom_port port) {
-  int i;
-  for (i = 0; i < n; i++) {
-    server *s = table[i];
-    if (s && strcmp(s->srv, name) == 0 &&
-        s->port == port) /* FIXME should use IP address */
-      return s;
-  }
-  return nullptr;
-}
-
 void update_servers(site_def *s, cargo_type operation) {
   u_int n;
 
