@@ -28,6 +28,7 @@
 #include <stdexcept>
 #include <string>
 #include <system_error>
+#include <type_traits>
 
 #ifdef _WIN32
 #include <WinSock2.h>
@@ -37,7 +38,6 @@
 #include <mysqld_error.h>
 #include <openssl/ssl.h>
 
-#include "duk_logging.h"
 #include "duk_module_shim.h"
 #include "duk_node_fs.h"
 #include "duktape.h"
@@ -46,12 +46,85 @@
 #include "mysql/harness/logging/logging.h"
 #include "mysql/harness/stdx/expected.h"
 #include "mysqlrouter/classic_protocol.h"
+#include "mysqlrouter/classic_protocol_constants.h"
+#include "mysqlrouter/classic_protocol_session_track.h"
 #include "statement_reader.h"
 
 IMPORT_LOG_FUNCTIONS()
 
 namespace server_mock {
 
+template <class T>
+constexpr uint8_t type_byte() {
+  return classic_protocol::Codec<T>::type_byte();
+}
+
+template <class T>
+std::string encode_session_tracker(const T &tracker) {
+  auto shared_caps = classic_protocol::capabilities::session_track;
+
+  std::string track_field{};
+  std::string session_change{};
+
+  {
+    auto encode_res = classic_protocol::encode(
+        tracker, shared_caps, net::dynamic_buffer(track_field));
+    if (!encode_res) {
+      //
+      return {};
+    }
+  }
+
+  {
+    auto encode_res = classic_protocol::encode(
+        classic_protocol::session_track::Field{type_byte<T>(), track_field},
+        shared_caps, net::dynamic_buffer(session_change));
+    if (!encode_res) {
+      //
+      return {};
+    }
+  }
+
+  return session_change;
+}
+
+std::string encode_session_trackers(
+    const std::vector<session_tracker_field> &trackers) {
+  std::string session_changes{};
+  for (const auto &tracker : trackers) {
+    if (std::holds_alternative<
+            classic_protocol::session_track::TransactionCharacteristics>(
+            tracker)) {
+      session_changes += encode_session_tracker(
+          std::get<classic_protocol::session_track::TransactionCharacteristics>(
+              tracker));
+    } else if (std::holds_alternative<
+                   classic_protocol::session_track::TransactionState>(
+                   tracker)) {
+      session_changes += encode_session_tracker(
+          std::get<classic_protocol::session_track::TransactionState>(tracker));
+    } else if (std::holds_alternative<
+                   classic_protocol::session_track::SystemVariable>(tracker)) {
+      session_changes += encode_session_tracker(
+          std::get<classic_protocol::session_track::SystemVariable>(tracker));
+    } else if (std::holds_alternative<classic_protocol::session_track::Schema>(
+                   tracker)) {
+      session_changes += encode_session_tracker(
+          std::get<classic_protocol::session_track::Schema>(tracker));
+    } else if (std::holds_alternative<classic_protocol::session_track::Gtid>(
+                   tracker)) {
+      session_changes += encode_session_tracker(
+          std::get<classic_protocol::session_track::Gtid>(tracker));
+    } else if (std::holds_alternative<classic_protocol::session_track::State>(
+                   tracker)) {
+      session_changes += encode_session_tracker(
+          std::get<classic_protocol::session_track::State>(tracker));
+    }
+  }
+
+  return session_changes;
+}
+
 std::unique_ptr<StatementReaderBase>
 DuktapeStatementReaderFactory::operator()() {
   try {
@@ -471,10 +544,11 @@ struct DuktapeStatementReader::Pimpl {
   }
 
   template <class INT_TYPE>
-  typename std::enable_if<std::is_unsigned<INT_TYPE>::value, INT_TYPE>::type
-  get_object_integer_value(duk_idx_t idx, const std::string &field,
-                           const INT_TYPE default_val = 0,
-                           bool is_required = false) {
+  INT_TYPE get_object_integer_value(duk_idx_t idx, const std::string &field,
+                                    const INT_TYPE default_val = 0,
+                                    bool is_required = false)
+    requires(std::is_unsigned_v<INT_TYPE>)
+  {
     INT_TYPE value;
 
     duk_get_prop_string(ctx, idx, field.c_str());
@@ -507,6 +581,78 @@ struct DuktapeStatementReader::Pimpl {
     return value;
   }
 
+  std::vector<session_tracker_field> get_session_trackers(duk_idx_t idx) {
+    std::vector<session_tracker_field> trackers;
+
+    duk_get_prop_string(ctx, idx, "session_trackers");
+    if (duk_is_array(ctx, -1)) {
+      // iterate over the column meta
+      duk_enum(ctx, -1, DUK_ENUM_ARRAY_INDICES_ONLY);
+      while (duk_next(ctx, -1, 1)) {
+        // @-2 ndx
+        // @-1 tracker
+
+        auto type = get_object_string_value(-1, "type");
+        if (type == "system_variable") {
+          trackers.emplace_back(classic_protocol::session_track::SystemVariable(
+              get_object_string_value(-1, "name"),
+              get_object_string_value(-1, "value")));
+        } else if (type == "trx_characteristics") {
+          auto stmt = get_object_string_value(-1, "trx_stmt");
+
+          trackers.emplace_back(
+              classic_protocol::session_track::TransactionCharacteristics(
+                  stmt));
+        } else if (type == "trx_state") {
+          auto state = get_object_string_value(-1, "state");
+
+          if (state.size() != 8) {
+            throw std::runtime_error(
+                "session_tracket.state must be size==8, is " +
+                std::to_string(state.size()));
+          }
+
+          trackers.emplace_back(
+              classic_protocol::session_track::TransactionState(
+                  std::span<char, 8>(state)));
+        } else if (type == "schema") {
+          auto schema = get_object_string_value(-1, "schema");
+
+          trackers.emplace_back(
+              classic_protocol::session_track::Schema(schema));
+        } else if (type == "gtid") {
+          auto gtid = get_object_string_value(-1, "gtid");
+
+          trackers.emplace_back(classic_protocol::session_track::Gtid(0, gtid));
+        } else if (type == "state") {
+          auto state = get_object_string_value(-1, "state");
+
+          if (state.size() != 1) {
+            throw std::runtime_error(
+                "session_tracket.state must be size==1, is " +
+                std::to_string(state.size()));
+          }
+
+          trackers.emplace_back(
+              classic_protocol::session_track::State(state[0]));
+        } else {
+          throw std::runtime_error("unknown session_tracket.type = " + type);
+        }
+
+        duk_pop(ctx);  // tracker
+        duk_pop(ctx);  // ndx
+      }
+      duk_pop(ctx);  // enum
+    } else if (duk_is_undefined(ctx, -1)) {
+      // ok
+    } else {
+      throw std::runtime_error("expected .session_tracker to be an array");
+    }
+    duk_pop(ctx);
+
+    return trackers;
+  }
+
   OkResponse get_ok(duk_idx_t idx) {
     if (!duk_is_object(ctx, idx)) {
       throw std::runtime_error("expect an object");
@@ -515,7 +661,9 @@ struct DuktapeStatementReader::Pimpl {
     return {get_object_integer_value<uint32_t>(-1, "affected_rows", 0),
             get_object_integer_value<uint32_t>(-1, "last_insert_id", 0),
             get_object_integer_value<uint16_t>(-1, "status", 0),
-            get_object_integer_value<uint16_t>(-1, "warning_count", 0)};
+            get_object_integer_value<uint16_t>(-1, "warning_count", 0),
+            get_object_string_value(-1, "message", ""),
+            encode_session_trackers(get_session_trackers(-1))};
   }
 
   ErrorResponse get_error(duk_idx_t idx) {
@@ -605,6 +753,17 @@ struct DuktapeStatementReader::Pimpl {
 
     duk_pop(ctx);  // "rows"
 
+    response.end_of_rows.affected_rows(
+        get_object_integer_value<uint32_t>(-1, "affected_rows", 0));
+    response.end_of_rows.last_insert_id(
+        get_object_integer_value<uint32_t>(-1, "last_insert_id", 0));
+    response.end_of_rows.status_flags(
+        get_object_integer_value<uint16_t>(-1, "status", 0));
+    response.end_of_rows.warning_count(
+        get_object_integer_value<uint16_t>(-1, "warning_count", 0));
+    response.end_of_rows.session_changes(
+        encode_session_trackers(get_session_trackers(-1)));
+
     return response;
   }
 
@@ -752,30 +911,40 @@ static bool check_notices_section(duk_context *ctx) {
   return has_notices;
 }
 
+static void check_handshake_greeting_exectime(duk_context *ctx) {
+  if (duk_is_number(ctx, -1) != 0 || duk_is_undefined(ctx, -1) != 0) {
+    return;
+  }
+
+  throw std::runtime_error("exec_time must be a number, if set. Is " +
+                           duk_get_type_names(ctx, -1));
+}
+
+static void check_handshake_object(duk_context *ctx) {
+  duk_get_prop_string(ctx, -1, "greeting");
+  if (duk_is_object(ctx, -1)) {
+    duk_get_prop_string(ctx, -1, "exec_time");
+    check_handshake_greeting_exectime(ctx);
+    duk_pop(ctx);
+  } else if (duk_is_undefined(ctx, -1)) {
+    // ok
+  } else {
+    throw std::runtime_error(
+        "handshake.greeting must be an object, if set. Is " +
+        duk_get_type_names(ctx, -1));
+  }
+  duk_pop(ctx);
+}
+
 static void check_handshake_section(duk_context *ctx) {
   duk_get_prop_string(ctx, -1, "handshake");
-  if (!duk_is_undefined(ctx, -1)) {
-    if (!duk_is_object(ctx, -1)) {
-      throw std::runtime_error("handshake must be an object, if set. Is " +
-                               duk_get_type_names(ctx, -1));
-    }
-    duk_get_prop_string(ctx, -1, "greeting");
-    if (!duk_is_undefined(ctx, -1)) {
-      if (!duk_is_object(ctx, -1)) {
-        throw std::runtime_error(
-            "handshake.greeting must be an object, if set. Is " +
-            duk_get_type_names(ctx, -1));
-      }
-      duk_get_prop_string(ctx, -1, "exec_time");
-      if (!duk_is_undefined(ctx, -1)) {
-        if (!duk_is_number(ctx, -1)) {
-          throw std::runtime_error("exec_time must be a number, if set. Is " +
-                                   duk_get_type_names(ctx, -1));
-        }
-      }
-      duk_pop(ctx);
-    }
-    duk_pop(ctx);
+  if (duk_is_object(ctx, -1) != 0) {
+    check_handshake_object(ctx);
+  } else if (duk_is_callable(ctx, -1) != 0 || duk_is_undefined(ctx, -1) != 0) {
+    // ok
+  } else {
+    throw std::runtime_error("handshake must be an object, if set. Is " +
+                             duk_get_type_names(ctx, -1));
   }
   duk_pop(ctx);
 }
@@ -803,10 +972,7 @@ DuktapeStatementReader::~DuktapeStatementReader() {
   DukHeapPool::instance()->release(std::move(pimpl_->heap_));
 }
 
-stdx::expected<classic_protocol::message::server::Greeting, std::error_code>
-DuktapeStatementReader::server_greeting(bool with_tls) {
-  auto *ctx = pimpl_->ctx;
-
+static classic_protocol::message::server::Greeting default_server_greeting() {
   // defaults
   std::string server_version = "8.0.23-mock";
   uint32_t connection_id = 0;
@@ -839,49 +1005,12 @@ DuktapeStatementReader::server_greeting(bool with_tls) {
       // compress_zstd (not yet)
       ;
 
-  if (with_tls) {
-    server_capabilities |= classic_protocol::capabilities::ssl;
-  }
-
   uint16_t status_flags = 0;
   uint8_t character_set = 0;
   std::string auth_method = MySQLNativePassword::name;
   std::string nonce = "01234567890123456789";
 
-  duk_get_prop_string(ctx, -1, "handshake");
-  if (!duk_is_undefined(ctx, -1)) {
-    if (!duk_is_object(ctx, -1)) {
-      throw std::runtime_error("handshake must be an object, if set. Is " +
-                               duk_get_type_names(ctx, -1));
-    }
-    duk_get_prop_string(ctx, -1, "greeting");
-    if (!duk_is_undefined(ctx, -1)) {
-      if (!duk_is_object(ctx, -1)) {
-        throw std::runtime_error(
-            "handshake.greeting must be an object, if set. Is " +
-            duk_get_type_names(ctx, -1));
-      }
-
-      server_version =
-          pimpl_->get_object_string_value(-1, "server_version", server_version);
-      connection_id = pimpl_->get_object_integer_value<uint32_t>(
-          -1, "connection_id", connection_id);
-      status_flags = pimpl_->get_object_integer_value<uint16_t>(
-          -1, "status_flags", status_flags);
-      character_set = pimpl_->get_object_integer_value<uint8_t>(
-          -1, "character_set", character_set);
-      server_capabilities = pimpl_->get_object_integer_value<uint32_t>(
-          -1, "capabilities", server_capabilities.to_ulong());
-      auth_method =
-          pimpl_->get_object_string_value(-1, "auth_method", auth_method);
-      nonce = pimpl_->get_object_string_value(-1, "nonce", nonce);
-    }
-    duk_pop(ctx);
-  }
-  duk_pop(ctx);
-
-  return {std::in_place,
-          0x0a,
+  return {0x0a,
           server_version,
           connection_id,
           nonce + std::string(1, '\0'),
@@ -891,50 +1020,60 @@ DuktapeStatementReader::server_greeting(bool with_tls) {
           auth_method};
 }
 
+stdx::expected<classic_protocol::message::server::Greeting, std::error_code>
+DuktapeStatementReader::server_greeting() {
+  auto *ctx = pimpl_->ctx;
+
+  auto greeting = default_server_greeting();
+
+  if (duk_is_object(ctx, -1)) {
+    greeting.version(pimpl_->get_object_string_value(-1, "server_version",
+                                                     greeting.version()));
+    greeting.connection_id(pimpl_->get_object_integer_value<uint32_t>(
+        -1, "connection_id", greeting.connection_id()));
+    greeting.status_flags(pimpl_->get_object_integer_value<uint16_t>(
+        -1, "status_flags", greeting.status_flags().to_ulong()));
+    greeting.collation(pimpl_->get_object_integer_value<uint8_t>(
+        -1, "character_set", greeting.collation()));
+    greeting.capabilities(pimpl_->get_object_integer_value<uint32_t>(
+        -1, "capabilities", greeting.capabilities().to_ulong()));
+    greeting.auth_method_name(pimpl_->get_object_string_value(
+        -1, "auth_method", greeting.auth_method_name()));
+    greeting.auth_method_data(pimpl_->get_object_string_value(
+        -1, "nonce", greeting.auth_method_data()));
+  }
+
+  return greeting;
+}
+
 std::chrono::microseconds DuktapeStatementReader::server_greeting_exec_time() {
   std::chrono::microseconds exec_time{};
 
   auto *ctx = pimpl_->ctx;
 
-  duk_get_prop_string(ctx, -1, "handshake");
-  if (!duk_is_undefined(ctx, -1)) {
-    if (!duk_is_object(ctx, -1)) {
-      throw std::runtime_error("handshake must be an object, if set. Is " +
-                               duk_get_type_names(ctx, -1));
-    }
-    duk_get_prop_string(ctx, -1, "greeting");
+  if (duk_is_object(ctx, -1)) {
+    duk_get_prop_string(ctx, -1, "exec_time");
     if (!duk_is_undefined(ctx, -1)) {
-      if (!duk_is_object(ctx, -1)) {
-        throw std::runtime_error(
-            "handshake.greeting must be an object, if set. Is " +
-            duk_get_type_names(ctx, -1));
+      if (!duk_is_number(ctx, -1)) {
+        throw std::runtime_error("exec_time must be a number, if set. Is " +
+                                 duk_get_type_names(ctx, -1));
+      }
+      if (duk_get_number(ctx, -1) < 0) {
+        throw std::out_of_range("exec_time must be a non-negative number");
       }
 
-      duk_get_prop_string(ctx, -1, "exec_time");
-      if (!duk_is_undefined(ctx, -1)) {
-        if (!duk_is_number(ctx, -1)) {
-          throw std::runtime_error("exec_time must be a number, if set. Is " +
-                                   duk_get_type_names(ctx, -1));
-        }
-        if (duk_get_number(ctx, -1) < 0) {
-          throw std::out_of_range("exec_time must be a non-negative number");
-        }
-
-        // exec_time is written in the tracefile as microseconds
-        exec_time = std::chrono::microseconds(
-            static_cast<long>(duk_get_number(ctx, -1) * 1000));
-      }
-      duk_pop(ctx);
+      // exec_time is written in the tracefile as microseconds
+      exec_time = std::chrono::microseconds(
+          static_cast<long>(duk_get_number(ctx, -1) * 1000));
     }
     duk_pop(ctx);
   }
-  duk_pop(ctx);
 
   return exec_time;
 }
 
 stdx::expected<DuktapeStatementReader::handshake_data, ErrorResponse>
-DuktapeStatementReader::handshake() {
+DuktapeStatementReader::handshake(bool is_greeting) {
   auto *ctx = pimpl_->ctx;
 
   std::optional<ErrorResponse> error;
@@ -944,17 +1083,46 @@ DuktapeStatementReader::handshake() {
   bool cert_required{false};
   std::optional<std::string> cert_issuer;
   std::optional<std::string> cert_subject;
-
+  stdx::expected<classic_protocol::message::server::Greeting, std::error_code>
+      server_greeting_res = default_server_greeting();
+  std::chrono::microseconds exec_time{};
   std::error_code ec{};
 
   duk_get_prop_string(ctx, -1, "handshake");
-  if (duk_is_object(ctx, -1)) {
+  if (duk_is_object(ctx, -1) || duk_is_callable(ctx, -1)) {
+    if (duk_is_callable(ctx, -1)) {
+      // call it.
+      duk_push_boolean(ctx, is_greeting);
+      duk_call(ctx, 1);
+
+      if (!duk_is_object(ctx, -1)) {
+        duk_pop(ctx);  // retval
+        duk_pop(ctx);  // handshake
+
+        return stdx::unexpected(ErrorResponse{
+            2013, "handshake-function must return an object, if set", "HY000"});
+      }
+    }
+
     duk_get_prop_string(ctx, -1, "error");
     if (!duk_is_undefined(ctx, -1)) {
       error = pimpl_->get_error(-1);
     }
     duk_pop(ctx);
 
+    duk_get_prop_string(ctx, -1, "greeting");
+    if (duk_is_object(ctx, -1)) {
+      server_greeting_res = server_greeting();
+      exec_time = server_greeting_exec_time();
+    } else if (duk_is_undefined(ctx, -1)) {
+      // ok
+    } else {
+      throw std::runtime_error(
+          "handshake.greeting must be an object, if set. Is " +
+          duk_get_type_names(ctx, -1));
+    }
+    duk_pop(ctx);
+
     duk_get_prop_string(ctx, -1, "auth");
     if (duk_is_object(ctx, -1)) {
       duk_get_prop_literal(ctx, -1, "username");
@@ -1003,12 +1171,20 @@ DuktapeStatementReader::handshake() {
   }
   duk_pop(ctx);
 
-  if (ec) {
-    return stdx::make_unexpected(ErrorResponse{2013, "hmm", "HY000"});
+  if (error) {
+    return stdx::unexpected(*error);
+  }
+  if (!server_greeting_res) {
+    ec = server_greeting_res.error();
   }
 
-  return handshake_data{error,         username,     password,
-                        cert_required, cert_subject, cert_issuer};
+  if (ec) {
+    return stdx::unexpected(ErrorResponse{2013, ec.message(), "HY000"});
+  }
+
+  return handshake_data{
+      *server_greeting_res, username,    password, cert_required,
+      cert_subject,         cert_issuer, exec_time};
 }
 
 // @pre on the stack is an object
