@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2022, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2022, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,6 +27,7 @@
 
 #include <chrono>
 #include <deque>
+#include <initializer_list>
 #include <memory>
 #include <optional>
 #include <ratio>
@@ -38,7 +40,6 @@
 #include "classic_greeting_forwarder.h"  // ServerGreetor
 #include "classic_init_schema_sender.h"
 #include "classic_query_sender.h"
-#include "classic_quit_sender.h"
 #include "classic_reset_connection_sender.h"
 #include "classic_set_option_sender.h"
 #include "mysql/harness/logging/logging.h"
@@ -46,7 +47,10 @@
 #include "mysql_com.h"
 #include "mysqlrouter/classic_protocol_message.h"
 #include "mysqlrouter/connection_pool_component.h"
+#include "mysqlrouter/datatypes.h"
+#include "mysqlrouter/utils.h"  // to_string
 #include "router_require.h"
+#include "sql_value.h"  // sql_value_to_string
 
 IMPORT_LOG_FUNCTIONS()
 
@@ -190,8 +194,10 @@ class SelectSessionVariablesHandler : public QuerySender::Handler {
       for (; !session_variables_.empty(); session_variables_.pop_front()) {
         auto &node = session_variables_.front();
 
-        connection_->execution_context().system_variables().set(
-            std::move(node.first), std::move(node.second));
+        connection_->client_protocol().system_variables().set(node.first,
+                                                              node.second);
+        connection_->server_protocol().system_variables().set(node.first,
+                                                              node.second);
       }
     }
   }
@@ -215,13 +221,18 @@ class SelectSessionVariablesHandler : public QuerySender::Handler {
 
   bool something_failed_{false};
 
-  std::deque<std::pair<std::string, Value>> session_variables_;
+  std::deque<std::pair<std::string, std::optional<std::string>>>
+      session_variables_;
 };
 
 }  // namespace
 
 stdx::expected<Processor::Result, std::error_code> LazyConnector::process() {
   switch (stage()) {
+    case Stage::Init:
+      return init();
+    case Stage::FromStash:
+      return from_stash();
     case Stage::Connect:
       return connect();
     case Stage::Connected:
@@ -274,9 +285,7 @@ stdx::expected<Processor::Result, std::error_code> LazyConnector::process() {
       }
 
       // reset the seq-id of the server side as this is a new command.
-      if (connection()->server_protocol() != nullptr) {
-        connection()->server_protocol()->seq_id(0xff);
-      }
+      connection()->server_protocol().seq_id(0xff);
 
       trace_span_end(trace_event_connect_);
 
@@ -286,16 +295,77 @@ stdx::expected<Processor::Result, std::error_code> LazyConnector::process() {
   harness_assert_this_should_not_execute();
 }
 
+stdx::expected<Processor::Result, std::error_code> LazyConnector::init() {
+  connection()->current_server_mode(connection()->expected_server_mode());
+
+  stage(Stage::FromStash);
+  return Result::Again;
+}
+
+stdx::expected<Processor::Result, std::error_code> LazyConnector::from_stash() {
+  connection()->has_transient_error_at_connect(false);
+
+  trace_event_connect_ =
+      trace_span(parent_event_, "mysql/prepare_server_connection");
+
+  if (!connection()->server_conn().is_open()) {
+    if (auto ep = connection()->destination_endpoint()) {
+      if (auto &tr = tracer()) {
+        tr.trace(Tracer::Event().stage("connect::from_stash"));
+      }
+
+      auto &pool_comp = ConnectionPoolComponent::get_instance();
+
+      if (auto pool =
+              pool_comp.get(ConnectionPoolComponent::default_pool_name())) {
+        TraceEvent *trace_event_from_stash = nullptr;
+
+        if (auto *ev = trace_event_connect_) {
+          trace_event_from_stash = trace_span(ev, "mysql/from_stash");
+        }
+
+        if (auto pop_res =
+                pool->unstash_mine(mysqlrouter::to_string(*ep), connection())) {
+          connection()->server_conn() = std::move(*pop_res);
+
+          // reset the seq-id of the server side as this is a new command.
+          connection()->server_protocol().seq_id(0xff);
+
+          if (auto &tr = tracer()) {
+            tr.trace(Tracer::Event().stage(
+                "connect::from_stash::unstashed::mine: fd=" +
+                std::to_string(connection()->server_conn().native_handle()) +
+                ", " + connection()->server_conn().endpoint()));
+          }
+
+          if (auto *ev = trace_event_from_stash) {
+            trace_set_connection_attributes(ev);
+            trace_span_end(ev);
+          }
+
+          stage(Stage::SetVars);
+          return Result::Again;
+        }
+
+        if (auto *ev = trace_event_from_stash) {
+          ev->attrs.emplace_back("mysql.error_message", "no match");
+          trace_span_end(ev, TraceEvent::StatusCode::kError);
+        }
+      }
+    }
+  }
+
+  stage(Stage::Connect);
+
+  return Result::Again;
+}
+
 stdx::expected<Processor::Result, std::error_code> LazyConnector::connect() {
   if (auto &tr = tracer()) {
     tr.trace(Tracer::Event().stage("connect::connect"));
   }
 
-  trace_event_connect_ =
-      trace_span(parent_event_, "mysql/prepare_server_connection");
-
-  auto *socket_splicer = connection()->socket_splicer();
-  auto &server_conn = socket_splicer->server_conn();
+  auto &server_conn = connection()->server_conn();
 
   if (!server_conn.is_open()) {
     stage(Stage::Connected);
@@ -318,10 +388,9 @@ stdx::expected<Processor::Result, std::error_code> LazyConnector::connect() {
  * the handshake part.
  */
 stdx::expected<Processor::Result, std::error_code> LazyConnector::connected() {
-  auto *socket_splicer = connection()->socket_splicer();
-  auto &server_conn = socket_splicer->server_conn();
-  auto *client_protocol = connection()->client_protocol();
-  auto *server_protocol = connection()->server_protocol();
+  auto &server_conn = connection()->server_conn();
+  auto &client_protocol = connection()->client_conn().protocol();
+  auto &server_protocol = server_conn.protocol();
 
   if (!server_conn.is_open()) {
     if (auto &tr = tracer()) {
@@ -344,13 +413,14 @@ stdx::expected<Processor::Result, std::error_code> LazyConnector::connected() {
   /*
    * if the connection is from the pool, we need a change user.
    */
-  if (server_protocol->server_greeting()) {
-    connection()->client_greeting_sent(true);
+  if (server_protocol.server_greeting()) {
+    connection()->server_conn().protocol().handshake_state(
+        ClassicProtocolState::HandshakeState::kFinished);
 
     if (!in_handshake_ &&
-        ((client_protocol->username() == server_protocol->username()) &&
-         (client_protocol->sent_attributes() ==
-          server_protocol->sent_attributes()))) {
+        ((client_protocol.username() == server_protocol.username()) &&
+         (client_protocol.sent_attributes() ==
+          server_protocol.sent_attributes()))) {
       // it is ok if the schema differs, it will be handled later set_schema()
 
       if (auto *ev = trace_event_authenticate_) {
@@ -365,10 +435,10 @@ stdx::expected<Processor::Result, std::error_code> LazyConnector::connected() {
         ev->attrs.emplace_back("mysql.remote.needs_full_handshake", true);
         ev->attrs.emplace_back(
             "mysql.remote.username_differs",
-            client_protocol->username() == server_protocol->username());
+            client_protocol.username() == server_protocol.username());
         ev->attrs.emplace_back("mysql.remote.connection_attributes_differ",
-                               client_protocol->sent_attributes() ==
-                                   server_protocol->sent_attributes());
+                               client_protocol.sent_attributes() ==
+                                   server_protocol.sent_attributes());
       }
 
       connection()->push_processor(std::make_unique<ChangeUserSender>(
@@ -387,10 +457,10 @@ stdx::expected<Processor::Result, std::error_code> LazyConnector::connected() {
         connection(), in_handshake_,
         [this](const classic_protocol::message::server::Error &err) {
           if (connect_error_is_transient(err) &&
-              (connection()->client_protocol()->password().has_value() ||
+              (connection()->client_protocol().password().has_value() ||
                !connection()
                     ->server_protocol()
-                    ->server_greeting()
+                    .server_greeting()
                     .has_value()) &&
               std::chrono::steady_clock::now() <
                   started_ + connection()->context().connect_retry_timeout()) {
@@ -422,7 +492,7 @@ stdx::expected<Processor::Result, std::error_code> LazyConnector::connected() {
 stdx::expected<Processor::Result, std::error_code>
 LazyConnector::authenticated() {
   if (!connection()->authenticated() ||
-      !connection()->socket_splicer()->server_conn().is_open()) {
+      !connection()->server_conn().is_open()) {
     if (auto &tr = tracer()) {
       tr.trace(Tracer::Event().stage("connect::authenticate::error"));
     }
@@ -434,6 +504,8 @@ LazyConnector::authenticated() {
     if (retry_connect_) {
       retry_connect_ = false;
 
+      connection()->has_transient_error_at_connect(true);
+
       stage(Stage::Connect);
       connection()->connect_timer().expires_after(kConnectRetryInterval);
       connection()->connect_timer().async_wait([this](std::error_code ec) {
@@ -462,30 +534,22 @@ LazyConnector::authenticated() {
 }
 
 namespace {
-void set_session_var(std::string &q, const std::string &key, const Value &val) {
+void set_session_var(std::string &q, const std::string &key,
+                     const std::optional<std::string> &val) {
   if (q.empty()) {
     q = "SET ";
   } else {
     q += ",\n    ";
   }
 
-  q += "@@SESSION." + key + " = " + val.to_string();
+  q += "@@SESSION." + key + " = " + sql_value_to_string(val);
 }
 
-void set_session_var_if_not_set(
-    std::string &q, const ExecutionContext::SystemVariables &sysvars,
-    const std::string &key, const Value &value) {
-  if (sysvars.get(key) == Value(std::nullopt)) {
-    set_session_var(q, key, value);
-  }
-}
-
-void set_session_var_or_value(std::string &q,
-                              const ExecutionContext::SystemVariables &sysvars,
-                              const std::string &key,
-                              const Value &default_value) {
+void set_session_var_or_value(
+    std::string &q, const ClassicProtocolState::SystemVariables &sysvars,
+    const std::string &key, const std::optional<std::string> &default_value) {
   auto value = sysvars.get(key);
-  if (value == Value(std::nullopt)) {
+  if (value == std::nullopt) {
     set_session_var(q, key, default_value);
   } else {
     set_session_var(q, key, value);
@@ -494,7 +558,8 @@ void set_session_var_or_value(std::string &q,
 }  // namespace
 
 stdx::expected<Processor::Result, std::error_code> LazyConnector::set_vars() {
-  auto &sysvars = connection()->execution_context().system_variables();
+  auto &client_sysvars = connection()->client_protocol().system_variables();
+  auto &server_sysvars = connection()->server_protocol().system_variables();
 
   std::string stmt;
 
@@ -504,32 +569,50 @@ stdx::expected<Processor::Result, std::error_code> LazyConnector::set_vars() {
 
   // must be first, to track all variables that are set.
   if (need_session_trackers) {
-    set_session_var_or_value(stmt, sysvars, "session_track_system_variables",
-                             Value("*"));
+    auto server_sysvar_has =
+        server_sysvars.find("session_track_system_variables");
+
+    if (!server_sysvar_has || *server_sysvar_has != "*") {
+      set_session_var_or_value(stmt, client_sysvars,
+                               "session_track_system_variables", "*");
+    }
   } else {
-    auto var = sysvars.get("session_track_system_variables");
-    if (var != Value(std::nullopt)) {
+    auto var = client_sysvars.get("session_track_system_variables");
+    if (var != std::nullopt) {
       set_session_var(stmt, "session_track_system_variables", var);
     }
   }
 
-  for (const auto &var : sysvars) {
+  for (const auto &var : client_sysvars) {
     // already set earlier.
     if (var.first == "session_track_system_variables") continue;
 
     // is read-only
     if (var.first == "statement_id") continue;
+    auto server_sysvar_has = server_sysvars.find(var.first);
 
-    set_session_var(stmt, var.first, var.second);
+    if (!server_sysvar_has || *server_sysvar_has != var.second) {
+      set_session_var(stmt, var.first, var.second);
+    }
   }
 
   if (need_session_trackers) {
-    set_session_var_if_not_set(stmt, sysvars, "session_track_gtids",
-                               Value("OWN_GTID"));
-    set_session_var_if_not_set(stmt, sysvars, "session_track_transaction_info",
-                               Value("CHARACTERISTICS"));
-    set_session_var_if_not_set(stmt, sysvars, "session_track_state_change",
-                               Value("ON"));
+    for (auto [k, v] : std::initializer_list<
+             std::pair<std::string, std::optional<std::string>>>{
+             {"session_track_gtids", "OWN_GTID"},
+             {"session_track_schema", "ON"},
+             {"session_track_state_change", "ON"},
+             {"session_track_transaction_info", "CHARACTERISTICS"},
+         }) {
+      auto client_sysvar_has = client_sysvars.find(k);
+      if (client_sysvar_has) continue;  // already handled above.
+
+      auto server_sysvar_has = server_sysvars.find(k);
+
+      if (!server_sysvar_has || *server_sysvar_has != v) {
+        set_session_var(stmt, k, v);
+      }
+    }
   }
 
   if (!stmt.empty()) {
@@ -541,13 +624,13 @@ stdx::expected<Processor::Result, std::error_code> LazyConnector::set_vars() {
 
     trace_event_set_vars_ = trace_span(trace_event_connect_, "mysql/set_var");
     if (auto *ev = trace_event_set_vars_) {
-      for (const auto &var : sysvars) {
+      for (const auto &var : client_sysvars) {
         if (var.first == "statement_id") continue;
 
-        if (var.second.value()) {
+        if (var.second) {
           ev->attrs.emplace_back(
               "mysql.session.@@SESSION." + var.first,
-              TraceEvent::element_type::second_type{*var.second.value()});
+              TraceEvent::element_type::second_type{*var.second});
         } else {
           // NULL
           ev->attrs.emplace_back(var.first,
@@ -580,12 +663,12 @@ LazyConnector::set_vars_done() {
 
 stdx::expected<Processor::Result, std::error_code>
 LazyConnector::set_server_option() {
-  auto *src_protocol = connection()->client_protocol();
-  auto *dst_protocol = connection()->server_protocol();
+  auto &src_protocol = connection()->client_protocol();
+  auto &dst_protocol = connection()->server_protocol();
 
-  bool client_has_multi_statements = src_protocol->client_capabilities().test(
+  bool client_has_multi_statements = src_protocol.client_capabilities().test(
       classic_protocol::capabilities::pos::multi_statements);
-  bool server_has_multi_statements = dst_protocol->client_capabilities().test(
+  bool server_has_multi_statements = dst_protocol.client_capabilities().test(
       classic_protocol::capabilities::pos::multi_statements);
 
   if (client_has_multi_statements == server_has_multi_statements) {
@@ -632,8 +715,7 @@ LazyConnector::fetch_sys_vars() {
     // fetch the sys-vars that aren't known yet.
     for (const auto &expected_var :
          {"collation_connection", "character_set_client", "sql_mode"}) {
-      const auto &sys_vars =
-          connection()->execution_context().system_variables();
+      const auto &sys_vars = connection()->client_protocol().system_variables();
       auto find_res = sys_vars.find(expected_var);
       if (!find_res) {
         if (oss.tellp() != 0) {
@@ -680,8 +762,8 @@ LazyConnector::fetch_sys_vars_done() {
 }
 
 stdx::expected<Processor::Result, std::error_code> LazyConnector::set_schema() {
-  auto client_schema = connection()->client_protocol()->schema();
-  auto server_schema = connection()->server_protocol()->schema();
+  auto client_schema = connection()->client_conn().protocol().schema();
+  auto server_schema = connection()->server_conn().protocol().schema();
 
   if (!client_schema.empty() && (client_schema != server_schema)) {
     if (auto &tr = tracer()) {
@@ -732,7 +814,7 @@ LazyConnector::wait_gtid_executed() {
                                         // didn't wait.
 
   if (connection()->wait_for_my_writes() &&
-      (connection()->expected_server_mode() ==
+      (connection()->current_server_mode() ==
        mysqlrouter::ServerMode::ReadOnly)) {
     auto gtid_executed = connection()->gtid_at_least_executed();
     if (!gtid_executed.empty()) {
@@ -796,34 +878,20 @@ stdx::expected<Processor::Result, std::error_code>
 LazyConnector::pool_or_close() {
   stage(Stage::FallbackToWrite);
 
-  const auto pool_res = pool_server_connection();
-  if (!pool_res) return stdx::make_unexpected(pool_res.error());
-
-  const auto still_open = *pool_res;
-  if (still_open) {
-    if (auto &tr = tracer()) {
-      tr.trace(Tracer::Event().stage("connect::pooled"));
-    }
-
-  } else {
-    // connection wasn't pooled as the pool was full. close it.
-    if (auto &tr = tracer()) {
-      tr.trace(Tracer::Event().stage("connect::pool_full"));
-    }
-
-    connection()->push_processor(std::make_unique<QuitSender>(connection()));
-  }
+  connection()->stash_server_conn();
 
   return Result::Again;
 }
 
 stdx::expected<Processor::Result, std::error_code>
 LazyConnector::fallback_to_write() {
-  if (already_fallback_ || connection()->expected_server_mode() ==
-                               mysqlrouter::ServerMode::ReadWrite) {
+  if (already_fallback_ ||
+      (connection()->expected_server_mode() ==
+       mysqlrouter::ServerMode::ReadWrite) ||
+      (connection()->current_server_mode() ==
+       mysqlrouter::ServerMode::ReadWrite)) {
     // only fallback to the primary once and if the client is asking for
     // "read-only" nodes
-    //
 
     // failed() is already set.
 
@@ -835,7 +903,8 @@ LazyConnector::fallback_to_write() {
     tr.trace(Tracer::Event().stage("connect::fallback_to_write"));
   }
 
-  connection()->expected_server_mode(mysqlrouter::ServerMode::ReadWrite);
+  // connect to the read-write node in read-only mode.
+  connection()->current_server_mode(mysqlrouter::ServerMode::ReadWrite);
   already_fallback_ = true;
 
   // reset the failed state
@@ -844,7 +913,7 @@ LazyConnector::fallback_to_write() {
   // the fallback will create a new trace-event
   trace_span_end(trace_event_connect_);
 
-  stage(Stage::Connect);
+  stage(Stage::FromStash);
   return Result::Again;
 }
 
@@ -955,7 +1024,7 @@ LazyConnector::fetch_user_attrs_done() {
   }
 
   auto enforce_res =
-      RouterRequire::enforce(connection()->socket_splicer()->client_channel(),
+      RouterRequire::enforce(connection()->client_conn().channel(),
                              *required_connection_attributes_fetcher_result_);
   if (!enforce_res) {
     failed(classic_protocol::message::server::Error{1045, "Access denied",
@@ -970,9 +1039,9 @@ LazyConnector::fetch_user_attrs_done() {
 
 stdx::expected<Processor::Result, std::error_code>
 LazyConnector::send_auth_ok() {
-  auto *socket_splicer = connection()->socket_splicer();
-  auto *dst_channel = socket_splicer->client_channel();
-  auto *dst_protocol = connection()->client_protocol();
+  auto &dst_conn = connection()->client_conn();
+  auto &dst_channel = dst_conn.channel();
+  auto &dst_protocol = dst_conn.protocol();
 
   if (!in_handshake_) {
     stage(Stage::Done);
@@ -986,7 +1055,7 @@ LazyConnector::send_auth_ok() {
   // tell the client that everything is ok.
   auto send_res =
       ClassicFrame::send_msg<classic_protocol::borrowed::message::server::Ok>(
-          dst_channel, dst_protocol, {0, 0, dst_protocol->status_flags(), 0});
+          dst_channel, dst_protocol, {0, 0, dst_protocol.status_flags(), 0});
   if (!send_res) return stdx::unexpected(send_res.error());
 
   stage(Stage::Done);
