@@ -251,6 +251,11 @@ var status_vars = {
               name: "session_track_state_change",
               value: "ON"
             },
+            {
+              type: "system_variable",
+              name: "session_track_schema",
+              value: "ON"
+            },
             {
               type: "trx_characteristics",
               value: "",
@@ -258,6 +263,30 @@ var status_vars = {
           ]
         }
       };
+    } else if (
+        stmt.indexOf('SET @@SESSION.character_set_client = "utf8mb4"') !== -1) {
+      // the trackers that are needed for connection-sharing.
+      return {
+        ok: {
+          session_trackers: [
+            {
+              type: "system_variable",
+              name: "character_set_client",
+              value: "utf8mb4",
+            },
+            {
+              type: "system_variable",
+              name: "collation_connection",
+              value: "utf8mb4_0900_ai_ci",
+            },
+            {
+              type: "system_variable",
+              name: "sql_mode",
+              value: "bar",
+            },
+          ]
+        }
+      };
     } else if (stmt === "DO 1") {
       return {ok: {}};
     } else if (stmt === "INSERT INTO testing.t1 VALUES ()") {
@@ -307,7 +336,7 @@ var status_vars = {
             },
             {
               type: "trx_state",
-              state: "________",
+              state: "T_______",
             },
           ]
         }
@@ -328,7 +357,7 @@ var status_vars = {
             },
             {
               type: "trx_state",
-              state: "________",
+              state: "T_______",
             },
           ]
         }
