@@ -1,15 +1,16 @@
-/* Copyright (c) 2013, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -594,6 +595,7 @@ int Mts_submode_logical_clock::schedule_next_event(Relay_log_info *rli,
   switch (ev->get_type_code()) {
     case mysql::binlog::event::GTID_LOG_EVENT:
     case mysql::binlog::event::ANONYMOUS_GTID_LOG_EVENT:
+    case mysql::binlog::event::GTID_TAGGED_LOG_EVENT:
       // TODO: control continuity
       ptr_group->sequence_number = sequence_number =
           static_cast<Gtid_log_event *>(ev)->sequence_number;
@@ -808,13 +810,13 @@ void Mts_submode_logical_clock::attach_temp_tables(THD *thd,
       cur_table->next = nullptr;
       mts_move_temp_tables_to_thd(thd, cur_table);
     } else
-        /* We must shift the C->temp_table pointer to the fist table unused in
-           this iteration. If all the tables have ben used C->temp_tables will
-           point to NULL */
-        if (!shifted) {
-      c_rli->info_thd->temporary_tables = cur_table;
-      shifted = true;
-    }
+      /* We must shift the C->temp_table pointer to the fist table unused in
+         this iteration. If all the tables have ben used C->temp_tables will
+         point to NULL */
+      if (!shifted) {
+        c_rli->info_thd->temporary_tables = cur_table;
+        shifted = true;
+      }
   } while (table);
   mysql_mutex_unlock(&c_rli->mts_temp_table_LOCK);
 }
