@@ -1,15 +1,16 @@
-/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -195,7 +196,7 @@ struct Slave_job_group {
   char *checkpoint_relay_log_name;
   std::atomic<int32> done;  // Flag raised by W,  read and reset by Coordinator
   ulong shifted;            // shift the last CP bitmap at receiving a new CP
-  time_t ts;                // Group's timestamp to update Seconds_behind_master
+  time_t ts;                // Group's timestamp to update Seconds_behind_source
 #ifndef NDEBUG
   bool notified{false};  // to debug group_master_log_name change notification
 #endif
@@ -595,14 +596,14 @@ class Slave_worker : public Relay_log_info {
     ERROR_LEAVING = 2,  // is set by Worker
     STOP = 3,           // is set by Coordinator upon receiving STOP
     STOP_ACCEPTED =
-        4  // is set by worker upon completing job when STOP SLAVE is issued
+        4  // is set by worker upon completing job when STOP REPLICA is issued
   };
 
   /*
     This function is used to make a copy of the worker object before we
-    destroy it on STOP SLAVE. This new object is then used to report the
-    worker status until next START SLAVE following which the new worker objects
-    will be used.
+    destroy it on STOP REPLICA. This new object is then used to report the
+    worker status until next START REPLICA following which the new worker
+    objects will be used.
   */
   void copy_values_for_PFS(ulong worker_id, en_running_state running_status,
                            THD *worker_thd, const Error &last_error,
@@ -763,8 +764,8 @@ class Slave_worker : public Relay_log_info {
         gtid_next=automatic. In this scenario, there is no reason to execute a
         Format_description_log_event. So we generate an error.
       */
-      if (info_thd->variables.gtid_next.type == AUTOMATIC_GTID ||
-          info_thd->variables.gtid_next.type == UNDEFINED_GTID) {
+      if (info_thd->variables.gtid_next.is_automatic() ||
+          info_thd->variables.gtid_next.is_undefined()) {
         bool in_active_multi_stmt =
             info_thd->in_active_multi_stmt_transaction();
 
@@ -847,19 +848,21 @@ class Slave_worker : public Relay_log_info {
     transaction if it is allowed. Retry policy and logic is similar to
     single-threaded slave.
 
-    @param[in] start_relay_number The extension number of the relay log which
-                 includes the first event of the transaction.
     @param[in] start_relay_pos The offset of the transaction's first event.
+    @param[in] start_event_relay_log_name The name of the relay log which
+                 includes the first event of the transaction.
 
-    @param[in] end_relay_number The extension number of the relay log which
-               includes the last event it should retry.
     @param[in] end_relay_pos The offset of the last event it should retry.
+    @param[in] end_event_relay_log_name The name of the relay log which
+               includes the last event it should retry.
 
     @retval false if transaction succeeds (possibly after a number of retries)
     @retval true  if transaction fails
   */
-  bool retry_transaction(uint start_relay_number, my_off_t start_relay_pos,
-                         uint end_relay_number, my_off_t end_relay_pos);
+  bool retry_transaction(my_off_t start_relay_pos,
+                         const char *start_event_relay_log_name,
+                         my_off_t end_relay_pos,
+                         const char *end_event_relay_log_name);
 
   bool set_info_search_keys(Rpl_info_handler *to) override;
 
@@ -911,6 +914,11 @@ class Slave_worker : public Relay_log_info {
                  va_list v_args) const override
       MY_ATTRIBUTE((format(printf, 4, 0)));
 
+  void do_report(loglevel level, int err_code,
+                 const Gtid_specification *gtid_next, const char *msg,
+                 va_list v_args) const override
+      MY_ATTRIBUTE((format(printf, 5, 0)));
+
  private:
   ulong gaq_index;           // GAQ index of the current assignment
   ulonglong master_log_pos;  // event's cached log_pos for possible error report
@@ -927,13 +935,18 @@ class Slave_worker : public Relay_log_info {
   Slave_worker &operator=(const Slave_worker &info);
   Slave_worker(const Slave_worker &info);
   bool worker_sleep(ulong seconds);
-  bool read_and_apply_events(uint start_relay_number, my_off_t start_relay_pos,
-                             uint end_relay_number, my_off_t end_relay_pos);
+  bool read_and_apply_events(my_off_t start_relay_pos,
+                             const char *start_event_relay_log_name,
+                             my_off_t end_relay_pos,
+                             const char *end_event_relay_log_name);
   void assign_partition_db(Log_event *ev);
 
+ public:
+  /**
+    Set the flag the signals a deadlock to false
+  */
   void reset_commit_order_deadlock();
 
- public:
   /**
      Returns an array with the expected column numbers of the primary key
      fields of the table repository.
