@@ -1,15 +1,16 @@
-/* Copyright (c) 2002, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2002, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -955,12 +956,31 @@ class sp_head {
   /// Flags of LEX::enum_binlog_stmt_unsafe.
   uint32 unsafe_flags;
 
- public:
+ private:
   /**
     language component related state of this sp.
    */
   external_program_handle m_language_stored_program;
 
+ public:
+  /**
+   * @brief Get the external program handle object
+   *
+   * @return external_program_handle
+   */
+  external_program_handle get_external_program_handle();
+
+  /**
+   * @brief Set the external program handle object
+   *
+   * @param sp The new external program handle object.
+   *           Use nullptr to unset the current one.
+   *
+   * @return true on errors
+   * @return false on success
+   */
+  bool set_external_program_handle(external_program_handle sp);
+
   /**
      Initialize and parse an external routine
 
@@ -1005,6 +1025,15 @@ class sp_head {
   */
   bool execute_external_routine(THD *thd);
 
+  /**
+    Core function for executing the external routine.
+
+    @param thd                  Thread context.
+
+    @return Error status.
+  */
+  bool execute_external_routine_core(THD *thd);
+
   /**
     Perform a forward flow analysis in the generated code.
     Mark reachable instructions, for the optimizer.
