@@ -29,12 +29,12 @@
 #include <algorithm>
 #include <cmath>
 
-#include "m_ctype.h"
 #include "my_byteorder.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
+#include "mysql/strings/m_ctype.h"
 #include "storage/heap/heapdef.h"
 
 /*
