@@ -54,6 +54,8 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "ut0list.h"
 #include "ut0wqueue.h"
 
+struct CHARSET_INFO;
+
 /** The FTS optimize thread's work queue. */
 static ib_wqueue_t *fts_optimize_wq;
 
@@ -108,88 +110,103 @@ enum fts_msg_type_t {
 /** Compressed list of words that have been read from FTS INDEX
 that needs to be optimized. */
 struct fts_zip_t {
-  lint status; /*!< Status of (un)/zip operation */
+  /** Status of (un)/zip operation */
+  lint status;
 
-  ulint n_words; /*!< Number of words compressed */
+  /** Number of words compressed */
+  ulint n_words;
 
-  ulint block_sz; /*!< Size of a block in bytes */
+  /** Size of a block in bytes */
+  ulint block_sz;
 
-  ib_vector_t *blocks; /*!< Vector of compressed blocks */
+  /** Vector of compressed blocks */
+  ib_vector_t *blocks;
 
-  ib_alloc_t *heap_alloc; /*!< Heap to use for allocations */
+  /** Heap to use for allocations */
+  ib_alloc_t *heap_alloc;
 
-  ulint pos; /*!< Offset into blocks */
+  /** Offset into blocks */
+  ulint pos;
 
-  ulint last_big_block; /*!< Offset of last block in the
-                        blocks array that is of size
-                        block_sz. Blocks beyond this offset
-                        are of size FTS_MAX_WORD_LEN */
+  /** Offset of last block in the blocks array that is of size
+  block_sz. Blocks beyond this offset are of size FTS_MAX_WORD_LEN */
+  ulint last_big_block;
 
-  z_streamp zp; /*!< ZLib state */
+  /** ZLib state */
+  z_streamp zp;
 
-  /*!< The value of the last word read
-  from the FTS INDEX table. This is
-  used to discard duplicates */
+  /** The value of the last word read from the FTS INDEX table.
+  This is used to discard duplicates (UTF-8 string) */
+  fts_string_t word;
 
-  fts_string_t word; /*!< UTF-8 string */
+  /** Maximum number of words to read in one pass */
+  ulint max_words;
 
-  ulint max_words; /*!< maximum number of words to read
-                   in one pass */
+  /** Did the last pass compress all remaining words in index? */
+  bool index_scan_complete;
 };
 
 /** Prepared statemets used during optimize */
 struct fts_optimize_graph_t {
-  /*!< Delete a word from FTS INDEX */
+  /** Delete a word from FTS INDEX */
   que_t *delete_nodes_graph;
-  /*!< Insert a word into FTS INDEX */
+  /** Insert a word into FTS INDEX */
   que_t *write_nodes_graph;
-  /*!< COMMIT a transaction */
+  /** COMMIT a transaction */
   que_t *commit_graph;
-  /*!< Read the nodes from FTS_INDEX */
+  /** Read the nodes from FTS_INDEX */
   que_t *read_nodes_graph;
 };
 
 /** Used by fts_optimize() to store state. */
 struct fts_optimize_t {
-  trx_t *trx; /*!< The transaction used for all SQL */
+  /** The transaction used for all SQL */
+  trx_t *trx;
 
-  ib_alloc_t *self_heap; /*!< Heap to use for allocations */
+  /** Heap to use for allocations */
+  ib_alloc_t *self_heap;
 
-  char *name_prefix; /*!< FTS table name prefix */
+  /** FTS table name prefix */
+  char *name_prefix;
 
-  fts_table_t fts_index_table; /*!< Common table definition */
+  /** Common table definition */
+  fts_table_t fts_index_table;
 
-  /*!< Common table definition */
+  /** Common table definition */
   fts_table_t fts_common_table;
 
-  dict_table_t *table; /*!< Table that has to be queried */
+  /** Table that has to be queried */
+  dict_table_t *table;
 
-  dict_index_t *index; /*!< The FTS index to be optimized */
+  /** The FTS index to be optimized */
+  dict_index_t *index;
 
-  fts_doc_ids_t *to_delete; /*!< doc ids to delete, we check against
-                            this vector and purge the matching
-                            entries during the optimizing
-                            process. The vector entries are
-                            sorted on doc id */
+  /** doc ids to delete, we check against this vector and purge the
+  matching entries during the optimizing process. The vector entries
+  are sorted on doc id */
+  fts_doc_ids_t *to_delete;
 
-  ulint del_pos; /*!< Offset within to_delete vector,
-                 this is used to keep track of where
-                 we are up to in the vector */
+  /** Offset within to_delete vector, this is used to keep track of where
+  we are up to in the vector */
+  ulint del_pos;
 
-  bool done; /*!< true when optimize finishes */
+  /** true when optimize finishes */
+  bool done;
 
-  ib_vector_t *words; /*!< Word + Nodes read from FTS_INDEX,
-                      it contains instances of fts_word_t */
+  /** Word + Nodes read from FTS_INDEX, it contains instances of fts_word_t */
+  ib_vector_t *words;
 
-  fts_zip_t *zip; /*!< Words read from the FTS_INDEX */
+  /** Words read from the FTS_INDEX */
+  fts_zip_t *zip;
 
-  fts_optimize_graph_t /*!< Prepared statements used during */
-      graph;           /*optimize */
+  /** Prepared statements used during optimize */
+  fts_optimize_graph_t graph;
 
-  ulint n_completed; /*!< Number of FTS indexes that have
-                     been optimized */
+  /** Number of FTS indexes that have been optimized */
+  ulint n_completed;
+
+  /** BEING_DELETED list regenerated */
   bool del_list_regenerated;
-  /*!< BEING_DELETED list regenerated */
 };
 
 /** Used by the optimize, to keep state during compacting nodes. */
@@ -274,6 +291,12 @@ static const char *fts_end_delete_sql =
     "DELETE FROM $being_deleted;\n"
     "DELETE FROM $being_deleted_cache;\n";
 
+/** Set fts_zip_t::word to empty string. */
+static void fts_zip_clear_word(fts_zip_t &zip) {
+  zip.word.f_len = 0;
+  memset(zip.word.f_str, 0, FTS_MAX_WORD_LEN);
+}
+
 /** Initialize fts_zip_t. */
 static void fts_zip_initialize(
     fts_zip_t *zip) /*!< out: zip instance to initialize */
@@ -285,12 +308,13 @@ static void fts_zip_initialize(
 
   zip->last_big_block = 0;
 
-  zip->word.f_len = 0;
-  memset(zip->word.f_str, 0, FTS_MAX_WORD_LEN);
+  fts_zip_clear_word(*zip);
 
   ib_vector_reset(zip->blocks);
 
   memset(zip->zp, 0, sizeof(*zip->zp));
+
+  zip->index_scan_complete = false;
 }
 
 /** Create an instance of fts_zip_t.
@@ -558,7 +582,7 @@ static bool fts_zip_extract_bytes(fts_zip_t *zip, /*!< in: Zip state + data */
       if (zip->pos > 0) {
         ulint prev = zip->pos - 1;
 
-        ut_a(zip->pos < ib_vector_size(zip->blocks));
+        ut_a_lt(zip->pos, ib_vector_size(zip->blocks));
 
         ut::free(ib_vector_getp(zip->blocks, prev));
         ib_vector_set(zip->blocks, prev, &null);
@@ -632,14 +656,14 @@ static bool fts_zip_read_word(fts_zip_t *zip,     /*!< in: Zip state + data */
     return false;
   }
 
-  ut_a(len <= FTS_MAX_WORD_LEN);
+  ut_a_le(len, FTS_MAX_WORD_LEN);
 
   word->f_len = len;
   const bool have_word = fts_zip_extract_bytes(zip, word->f_str, len);
   ut_ad(have_word);
 
   word->f_str[len] = 0;
-  ut_ad(word->f_len == strlen((char *)word->f_str));
+  ut_ad_eq(word->f_len, strlen((char *)word->f_str));
 
   return have_word;
 }
@@ -649,19 +673,36 @@ static bool fts_zip_read_word(fts_zip_t *zip,     /*!< in: Zip state + data */
  @return false on EOF */
 static bool fts_fetch_index_words(
     void *row,      /*!< in: sel_node_t* */
-    void *user_arg) /*!< in: pointer to ib_vector_t */
+    void *user_arg) /*!< in: pointer to fts_optimize_t */
 {
-  sel_node_t *sel_node = static_cast<sel_node_t *>(row);
-  fts_zip_t *zip = static_cast<fts_zip_t *>(user_arg);
-  que_node_t *exp = sel_node->select_list;
-  dfield_t *dfield = que_node_get_val(exp);
+  const auto *const sel_node = static_cast<sel_node_t *>(row);
+  const auto optim = static_cast<fts_optimize_t *>(user_arg);
+  fts_zip_t *const zip = optim->zip;
+  que_node_t *const exp = sel_node->select_list;
+  const CHARSET_INFO *const charset = optim->fts_index_table.charset;
+  const dfield_t *const dfield = que_node_get_val(exp);
   ushort len = static_cast<ushort>(dfield_get_len(dfield));
-  void *data = dfield_get_data(dfield);
+  void *const data = dfield_get_data(dfield);
 
-  /* Skip the duplicate words. */
-  if (zip->word.f_len == static_cast<ulint>(len) &&
-      !memcmp(zip->word.f_str, data, len)) {
-    return true;
+  /* Ensure each class of collation-equal tokens is compressed only once. */
+  {
+    /* Compare current word with previous using collation. */
+    auto comp = my_strnncoll(charset, static_cast<uint8_t *>(data), len,
+                             zip->word.f_str, zip->word.f_len);
+    /* Duplicate detection is guaranteed by callback invocations being
+    grouped into sequences (corresponding to aux tables), where all
+    collation-equal words are in the same sequence; between these
+    sequences zip->word is reset to an empty string (every non-empty
+    word is assumed to be later in sort order than the empty string).
+    Within each sequence words are sorted (using collation) in ascending
+    order, guaranteeing all collation-equal words are passed to the
+    callback in consecutive calls, which allows us to eliminate
+    duplicates by comparing each word only with the one preceding it. */
+    ut_ad(comp >= 0);
+    /* Skip the duplicate words. */
+    if (comp == 0) {
+      return true;
+    }
   }
 
   ut_a(len <= FTS_MAX_WORD_LEN);
@@ -768,8 +809,6 @@ static void fts_zip_deflate_end(
 {
   pars_info_t *info;
   que_t *graph;
-  ulint selected;
-  fts_zip_t *zip = nullptr;
   dberr_t error = DB_SUCCESS;
   mem_heap_t *heap = static_cast<mem_heap_t *>(optim->self_heap->arg);
   bool inited = false;
@@ -782,16 +821,51 @@ static void fts_zip_deflate_end(
     fts_zip_initialize(optim->zip);
   }
 
-  for (selected = fts_select_index(optim->fts_index_table.charset, word->f_str,
-                                   word->f_len);
-       selected < FTS_NUM_AUX_INDEX; selected++) {
+  fts_zip_t *zip = optim->zip;
+
+  /* 'aux index' for these variables means 0-based ordinal of an FTS aux table;
+  each FTS index is represented by FTS_NUM_AUX_INDEX (6) aux tables:
+  *_aux_idx==0 means 1st table, 1 means 2nd table etc. */
+  auto selected_aux_idx = fts_select_index(optim->fts_index_table.charset,
+                                           word->f_str, word->f_len);
+
+  ut_a(optim->zip->n_words == 0);
+  ut_a(n_words > 0);
+  /* Iterate over aux tables starting with table for last word
+  previous tables were covered by previous OPTIMIZE passes.
+  Word limit is checked after fetching all words from an aux table:
+  this means we can go over the limit, but ensures that either all
+  instances of a given word were fetched, or none were. */
+  for (; selected_aux_idx < FTS_NUM_AUX_INDEX && optim->zip->n_words < n_words;
+       ++selected_aux_idx) {
     char table_name[MAX_FULL_NAME_LEN];
 
-    optim->fts_index_table.suffix = fts_get_suffix(selected);
+    optim->fts_index_table.suffix = fts_get_suffix(selected_aux_idx);
+
+    /* The list of words to be processed may not have any duplicates -
+    meaning here words that are equal in collation order. Elimination
+    of duplicates is performed inside the fts_fetch_index_words callback
+    using optim->zip->word as the previous word processed. It relies on
+    all collation-equal words being passed to the callback in consecutive
+    calls. This property holds because:
+    1. All words equal by collation order are indexed in the same aux
+       table,
+    2. Words from a given aux table are processed by collation
+       (ascending) order.
+    (1) is ensured by the way words are assigned to aux table number -
+    see fts_select_index function. (2) is guaranteed by 'ORDER BY word'
+    clause in the query below.
+
+    optim->zip->word is cleared out when moving from one aux table to
+    another; it is not needed, as there can be no duplicates between
+    tables, but doing so allows the callback to assert ordering within
+    the table. */
+
+    fts_zip_clear_word(*optim->zip);
 
     info = pars_info_create();
 
-    pars_info_bind_function(info, "my_func", fts_fetch_index_words, optim->zip);
+    pars_info_bind_function(info, "my_func", fts_fetch_index_words, optim);
 
     pars_info_bind_varchar_literal(info, "word", word->f_str, word->f_len);
 
@@ -857,11 +931,10 @@ static void fts_zip_deflate_end(
     }
 
     fts_que_graph_free(graph);
+  }
 
-    /* Check if max word to fetch is exceeded */
-    if (optim->zip->n_words >= n_words) {
-      break;
-    }
+  if (selected_aux_idx == FTS_NUM_AUX_INDEX) {
+    zip->index_scan_complete = true;
   }
 
   if (error == DB_SUCCESS && zip->status == Z_OK && zip->n_words > 0) {
@@ -960,7 +1033,7 @@ dberr_t fts_table_fetch_doc_ids(
   if (error == DB_SUCCESS) {
     fts_sql_commit(trx);
 
-    ib_vector_sort(doc_ids->doc_ids, fts_update_doc_id_cmp);
+    ib_vector_sort(doc_ids->doc_ids, fts_doc_id_field_cmp<fts_update_t>);
   } else {
     fts_sql_rollback(trx);
   }
@@ -1743,39 +1816,38 @@ static void fts_optimize_words(
  @return DB_SUCCESS if all OK */
 [[nodiscard]] static dberr_t fts_optimize_index_read_words(
     fts_optimize_t *optim, /*!< in: optimize instance */
-    dict_index_t *index,   /*!< in: table with one FTS index */
-    fts_string_t *word)    /*!< in: buffer to use */
+    dict_index_t *index)   /*!< in: table with one FTS index */
 {
   dberr_t error = DB_SUCCESS;
 
+  fts_string_t word;
+  byte str[FTS_MAX_WORD_LEN + 1];
+
+  /* We need to read the last word optimized so that we start from
+  the next word. */
+  word.f_str = str;
+
+  /* We set the length of word to the size of str since we
+  need to pass the max len info to the fts_get_config_value() function. */
+  word.f_len = sizeof(str) - 1;
+
   if (optim->del_list_regenerated) {
-    word->f_len = 0;
+    word.f_len = 0;
   } else {
     /* Get the last word that was optimized from
     the config table. */
     error = fts_config_get_index_value(optim->trx, index,
-                                       FTS_LAST_OPTIMIZED_WORD, word);
+                                       FTS_LAST_OPTIMIZED_WORD, &word);
   }
 
   /* If record not found then we start from the top. */
   if (error == DB_RECORD_NOT_FOUND) {
-    word->f_len = 0;
+    word.f_len = 0;
     error = DB_SUCCESS;
   }
 
-  while (error == DB_SUCCESS) {
-    error = fts_index_fetch_words(optim, word, fts_num_word_optimize);
-
-    if (error == DB_SUCCESS) {
-      /* Reset the last optimized word to '' if no
-      more words could be read from the FTS index. */
-      if (optim->zip->n_words == 0) {
-        word->f_len = 0;
-        *word->f_str = 0;
-      }
-
-      break;
-    }
+  if (error == DB_SUCCESS) {
+    error = fts_index_fetch_words(optim, &word, fts_num_word_optimize);
   }
 
   return (error);
@@ -1788,63 +1860,52 @@ static void fts_optimize_words(
     fts_optimize_t *optim, /*!< in: optimize instance */
     dict_index_t *index)   /*!< in: table with one FTS index */
 {
-  fts_string_t word;
-  dberr_t error;
-  byte str[FTS_MAX_WORD_LEN + 1];
-
   /* Set the current index that we have to optimize. */
   optim->fts_index_table.index_id = index->id;
   optim->fts_index_table.charset = fts_index_get_charset(index);
 
   optim->done = false; /* Optimize until !done */
 
-  /* We need to read the last word optimized so that we start from
-  the next word. */
-  word.f_str = str;
-
-  /* We set the length of word to the size of str since we
-  need to pass the max len info to the fts_get_config_value() function. */
-  word.f_len = sizeof(str) - 1;
-
-  memset(word.f_str, 0x0, word.f_len);
-
   /* Read the words that will be optimized in this pass. */
-  error = fts_optimize_index_read_words(optim, index, &word);
-
-  if (error == DB_SUCCESS) {
-    int zip_error;
-
-    ut_a(optim->zip->pos == 0);
-    ut_a(optim->zip->zp->total_in == 0);
-    ut_a(optim->zip->zp->total_out == 0);
-
-    zip_error = inflateInit(optim->zip->zp);
-    ut_a(zip_error == Z_OK);
-
-    word.f_len = 0;
-    word.f_str = str;
-
-    /* Read the first word to optimize from the Zip buffer. */
-    if (!fts_zip_read_word(optim->zip, &word)) {
-      optim->done = true;
-    } else {
-      fts_optimize_words(optim, index, &word);
-    }
-
-    /* If we couldn't read any records then optimize is
-    complete. Increment the number of indexes that have
-    been optimized and set FTS index optimize state to
-    completed. */
-    if (error == DB_SUCCESS && optim->zip->n_words == 0) {
-      error = fts_optimize_index_completed(optim, index);
-
-      if (error == DB_SUCCESS) {
-        ++optim->n_completed;
-      }
-    }
+  if (const auto error = fts_optimize_index_read_words(optim, index);
+      error != DB_SUCCESS) {
+    return error;
   }
 
-  return (error);
+  int zip_error;
+
+  ut_a(optim->zip->pos == 0);
+  ut_a(optim->zip->zp->total_in == 0);
+  ut_a(optim->zip->zp->total_out == 0);
+
+  zip_error = inflateInit(optim->zip->zp);
+  ut_a(zip_error == Z_OK);
+
+  /* Buffer for uncompressing words */
+  fts_string_t word;
+  byte str[FTS_MAX_WORD_LEN + 1];
+  word.f_len = 0;
+  word.f_str = str;
+
+  /* Read the first word to optimize from the Zip buffer. */
+  if (!fts_zip_read_word(optim->zip, &word)) {
+    optim->done = true;
+  } else {
+    fts_optimize_words(optim, index, &word);
+  }
+
+  /* If all remaining words in index have been read in this pass, perform
+  final actions to complete the index optimization. */
+  if (optim->zip->index_scan_complete) {
+    if (const auto error = fts_optimize_index_completed(optim, index);
+        error != DB_SUCCESS) {
+      return error;
+    }
+
+    ++optim->n_completed;
+  }
+
+  return DB_SUCCESS;
 }
 
 /** Delete the document ids in the delete, and delete cache tables.
