@@ -50,8 +50,9 @@ this program; if not, write to the Free Software Foundation, Inc.,
 @param[in]  end_ptr  buffer end
 @param[out] index    own: dummy index
 @return parsed record end, NULL if not a complete record */
-[[nodiscard]] static byte *mlog_parse_index_v1(byte *ptr, const byte *end_ptr,
-                                               dict_index_t **index);
+[[nodiscard]] static const byte *mlog_parse_index_v1(const byte *ptr,
+                                                     const byte *end_ptr,
+                                                     dict_index_t **index);
 
 /** Catenates n bytes to the mtr log.
 @param[in] mtr Mini-transaction
@@ -100,9 +101,10 @@ void mlog_write_initial_log_record(
 @param[out]     id              table id
 @param[out]     version         table dynamic metadata version
 @return parsed record end, NULL if not a complete record */
-byte *mlog_parse_initial_dict_log_record(const byte *ptr, const byte *end_ptr,
-                                         mlog_id_t *type, table_id_t *id,
-                                         uint64_t *version) {
+const byte *mlog_parse_initial_dict_log_record(const byte *ptr,
+                                               const byte *end_ptr,
+                                               mlog_id_t *type, table_id_t *id,
+                                               uint64_t *version) {
   if (end_ptr < ptr + 1) {
     return (nullptr);
   }
@@ -124,12 +126,12 @@ byte *mlog_parse_initial_dict_log_record(const byte *ptr, const byte *end_ptr,
 
   *version = mach_parse_u64_much_compressed(&ptr, end_ptr);
 
-  return (const_cast<byte *>(ptr));
+  return ptr;
 }
 
 /** Parses an initial log record written by mlog_write_initial_log_record.
  @return parsed record end, NULL if not a complete record */
-byte *mlog_parse_initial_log_record(
+const byte *mlog_parse_initial_log_record(
     const byte *ptr,     /*!< in: buffer */
     const byte *end_ptr, /*!< in: buffer end */
     mlog_id_t *type,     /*!< out: log record type: MLOG_1BYTE, ... */
@@ -155,12 +157,12 @@ byte *mlog_parse_initial_log_record(
     *page_no = mach_parse_compressed(&ptr, end_ptr);
   }
 
-  return (const_cast<byte *>(ptr));
+  return ptr;
 }
 
 /** Parses a log record written by mlog_write_ulint or mlog_write_ull.
  @return parsed record end, NULL if not a complete record or a corrupt record */
-byte *mlog_parse_nbytes(
+const byte *mlog_parse_nbytes(
     mlog_id_t type,      /*!< in: log record type: MLOG_1BYTE, ... */
     const byte *ptr,     /*!< in: buffer */
     const byte *end_ptr, /*!< in: buffer end */
@@ -202,7 +204,7 @@ byte *mlog_parse_nbytes(
       mach_write_to_8(page + offset, dval);
     }
 
-    return (const_cast<byte *>(ptr));
+    return ptr;
   }
 
   val = mach_parse_compressed(&ptr, end_ptr);
@@ -248,7 +250,7 @@ byte *mlog_parse_nbytes(
       ptr = nullptr;
   }
 
-  return (const_cast<byte *>(ptr));
+  return ptr;
 }
 
 /** Writes 1, 2 or 4 bytes to a file page. Writes the corresponding log
@@ -329,12 +331,15 @@ void mlog_write_string(byte *ptr,       /*!< in: pointer where to write */
                        ulint len,       /*!< in: string length */
                        mtr_t *mtr)      /*!< in: mini-transaction handle */
 {
-  ut_ad(ptr && mtr);
+  ut_ad(ptr);
+  ut_ad(mtr != nullptr || buf_page_t::is_memory(ptr));
   ut_a(len < UNIV_PAGE_SIZE);
 
   memcpy(ptr, str, len);
 
-  mlog_log_string(ptr, len, mtr);
+  if (mtr != nullptr) {
+    mlog_log_string(ptr, len, mtr);
+  }
 }
 
 /** Logs a write of a string to a file page buffered in the buffer pool.
@@ -369,11 +374,11 @@ void mlog_log_string(byte *ptr,  /*!< in: pointer written to */
 
 /** Parses a log record written by mlog_write_string.
  @return parsed record end, NULL if not a complete record */
-byte *mlog_parse_string(
-    byte *ptr,      /*!< in: buffer */
-    byte *end_ptr,  /*!< in: buffer end */
-    byte *page,     /*!< in: page where to apply the log record, or NULL */
-    void *page_zip) /*!< in/out: compressed page, or NULL */
+const byte *mlog_parse_string(
+    const byte *ptr,     /*!< in: buffer */
+    const byte *end_ptr, /*!< in: buffer end */
+    byte *page,          /*!< in: page where to apply the log record, or NULL */
+    void *page_zip)      /*!< in/out: compressed page, or NULL */
 {
   ulint offset;
   ulint len;
@@ -411,8 +416,8 @@ byte *mlog_parse_string(
   return (ptr + len);
 }
 
-byte *mlog_parse_index_8027(byte *ptr, const byte *end_ptr, bool comp,
-                            dict_index_t **index) {
+const byte *mlog_parse_index_8027(const byte *ptr, const byte *end_ptr,
+                                  bool comp, dict_index_t **index) {
   ulint i;
   dict_table_t *table;
   dict_index_t *ind;
@@ -515,12 +520,15 @@ constexpr size_t inst_col_info_size = 6;
 @param[in]   is_comp       true if COMP
 @param[in]   is_versioned  if table has row versions
 @param[in]   is_instant    true if table has INSTANT cols
+@param[in]   fields_with_changed_order bitmap to indicate fields with changed
+                                       order
 @param[out]  size_needed   total size needed on REDO LOG */
 static void log_index_get_size_needed(const dict_index_t *index, size_t size,
                                       uint16_t n, bool is_comp,
                                       bool is_versioned, bool is_instant,
+                                      const bool *fields_with_changed_order,
                                       size_t &size_needed) {
-  auto size_for_versioned_fields = [](const dict_index_t *ind) {
+  auto size_for_versioned_fields = [&](const dict_index_t *ind) {
     size_t _size = 0;
     /* 2 bytes for number of columns with version */
     _size += 2;
@@ -530,6 +538,16 @@ static void log_index_get_size_needed(const dict_index_t *index, size_t size,
     ut_ad(n_versioned_fields != 0);
 
     _size += n_versioned_fields * inst_col_info_size;
+
+    /* For fields with changed order */
+    size_t n_changed_order_fields = 0;
+    for (size_t i = 0; i < n; i++) {
+      if (fields_with_changed_order[i]) {
+        n_changed_order_fields++;
+      }
+    }
+    _size += n_changed_order_fields * inst_col_info_size;
+
     return (_size);
   };
 
@@ -799,9 +817,33 @@ bool mlog_open_and_write_index(mtr_t *mtr, const byte *rec,
     n = DICT_INDEX_SPATIAL_NODEPTR_SIZE;
   }
 
+  /* Ordinal position of an existing field can't be changed with INSTANT
+  algorithm. But when it is combined with ADD/DROP COLUMN, ordinal position
+  of a filed can be changed. This bool array of size #fields in index,
+  represents if ordinal position of an existing filed is changed. */
+  bool *fields_with_changed_order = nullptr;
+  if (is_versioned) {
+    fields_with_changed_order = new bool[n];
+    memset(fields_with_changed_order, false, (sizeof(bool) * n));
+
+    uint16_t phy_pos = 0;
+    for (size_t i = 0; i < n; i++) {
+      dict_field_t *field = index->get_field(i);
+      const dict_col_t *col = field->col;
+
+      if (col->is_instant_added() || col->is_instant_dropped()) {
+        continue;
+      } else if (field->get_phy_pos() >= phy_pos) {
+        phy_pos = field->get_phy_pos();
+      } else {
+        fields_with_changed_order[i] = true;
+      }
+    }
+  }
+
   size_t size_needed = 0;
   log_index_get_size_needed(index, size, n, is_comp, is_versioned, is_instant,
-                            size_needed);
+                            fields_with_changed_order, size_needed);
   size_t total = size_needed;
   size_t alloc = total;
   if (alloc > mtr_buf_t::MAX_DATA_SIZE) {
@@ -809,6 +851,9 @@ bool mlog_open_and_write_index(mtr_t *mtr, const byte *rec,
   }
 
   if (!mlog_open(mtr, alloc, log_ptr)) {
+    if (is_versioned) {
+      delete[] fields_with_changed_order;
+    }
     /* logging is disabled */
     return (false);
   }
@@ -846,30 +891,6 @@ bool mlog_open_and_write_index(mtr_t *mtr, const byte *rec,
     return true;
   };
 
-  /* Ordinal position of an existing field can't be changed with INSTANT
-  algorithm. But when it is combined with ADD/DROP COLUMN, ordinal position
-  of a filed can be changed. This bool array of size #fields in index,
-  represents if ordinal position of an existing filed is changed. */
-  bool *fields_with_changed_order = nullptr;
-  if (is_versioned) {
-    fields_with_changed_order = new bool[n];
-    memset(fields_with_changed_order, false, (sizeof(bool) * n));
-
-    uint16_t phy_pos = 0;
-    for (size_t i = 0; i < n; i++) {
-      dict_field_t *field = index->get_field(i);
-      const dict_col_t *col = field->col;
-
-      if (col->is_instant_added() || col->is_instant_dropped()) {
-        continue;
-      } else if (col->get_phy_pos() >= phy_pos) {
-        phy_pos = col->get_phy_pos();
-      } else {
-        fields_with_changed_order[i] = true;
-      }
-    }
-  }
-
   if (is_comp) {
     /* Write fields info. */
     if (!log_index_fields(index, n, is_versioned, instant_fields_to_log,
@@ -921,7 +942,8 @@ bool mlog_open_and_write_index(mtr_t *mtr, const byte *rec,
 @param[in]   ptr      pointer to buffer
 @param[in]   end_ptr  pointer to end of buffer
 @param[out]  val      read 2 bytes value */
-static byte *read_2_bytes(byte *ptr, const byte *end_ptr, uint16_t &val) {
+static const byte *read_2_bytes(const byte *ptr, const byte *end_ptr,
+                                uint16_t &val) {
   if (end_ptr < ptr + 2) {
     return (nullptr);
   }
@@ -934,7 +956,8 @@ static byte *read_2_bytes(byte *ptr, const byte *end_ptr, uint16_t &val) {
 @param[in]   ptr      pointer to buffer
 @param[in]   end_ptr  pointer to end of buffer
 @param[out]  val      read 2 bytes value */
-static byte *read_1_bytes(byte *ptr, const byte *end_ptr, uint8_t &val) {
+static const byte *read_1_bytes(const byte *ptr, const byte *end_ptr,
+                                uint8_t &val) {
   if (end_ptr < ptr + 1) {
     return (nullptr);
   }
@@ -953,10 +976,11 @@ static byte *read_1_bytes(byte *ptr, const byte *end_ptr, uint8_t &val) {
 @param[out]  n_uniq        n_uniq for index
 @param[out]  inst_cols     number of column before first instant add was done.
 @return pointer to buffer. */
-static byte *parse_index_column_counts(byte *ptr, const byte *end_ptr,
-                                       bool is_comp, bool is_versioned,
-                                       bool is_instant, uint16_t &n,
-                                       uint16_t &n_uniq, uint16_t &inst_cols) {
+static const byte *parse_index_column_counts(const byte *ptr,
+                                             const byte *end_ptr, bool is_comp,
+                                             bool is_versioned, bool is_instant,
+                                             uint16_t &n, uint16_t &n_uniq,
+                                             uint16_t &inst_cols) {
   if (!is_versioned && !is_comp) {
     n = n_uniq = 1;
     inst_cols = 0;
@@ -999,9 +1023,10 @@ static byte *parse_index_column_counts(byte *ptr, const byte *end_ptr,
 @param[in,out]   ind      dummy index
 @param[in,out]   table    dummy table
 @return pointer to log buffer */
-static byte *parse_index_fields(byte *ptr, const byte *end_ptr, uint16_t n,
-                                uint16_t n_uniq, bool is_versioned,
-                                dict_index_t *&ind, dict_table_t *&table) {
+static const byte *parse_index_fields(const byte *ptr, const byte *end_ptr,
+                                      uint16_t n, uint16_t n_uniq,
+                                      bool is_versioned, dict_index_t *&ind,
+                                      dict_table_t *&table) {
   for (size_t i = 0; i < n; i++) {
     /* For redundant, col len metadata isn't needed for recovery as it is
     part of record itself. */
@@ -1066,9 +1091,11 @@ using instant_fields_list_t = std::vector<Field_instant_info>;
 @param[out]  f         vector of fields with versions
 @param[out]  crv       current row version
 @param[out]  n_dropped number of dropped columns */
-static byte *parse_index_versioned_fields(byte *ptr, const byte *end_ptr,
-                                          instant_fields_list_t &f,
-                                          uint16_t &crv, size_t &n_dropped) {
+static const byte *parse_index_versioned_fields(const byte *ptr,
+                                                const byte *end_ptr,
+                                                instant_fields_list_t &f,
+                                                uint16_t &crv,
+                                                size_t &n_dropped) {
   uint16_t n_inst = 0;
   ptr = read_2_bytes(ptr, end_ptr, n_inst);
   if (ptr == nullptr) return (nullptr);
@@ -1178,21 +1205,24 @@ static void populate_dummy_fields(dict_index_t *index, dict_table_t *table,
       n;
 }
 
-static byte *parse_index_log_version(byte *ptr, const byte *end_ptr,
-                                     uint8_t &version) {
+static const byte *parse_index_log_version(const byte *ptr, const byte *end_ptr,
+                                           uint8_t &version) {
   ptr = read_1_bytes(ptr, end_ptr, version);
   if (ptr == nullptr) return nullptr;
 
   return ptr;
 }
-static byte *parse_index_flag(byte *ptr, const byte *end_ptr, uint8_t &flag) {
+
+static const byte *parse_index_flag(const byte *ptr, const byte *end_ptr,
+                                    uint8_t &flag) {
   ptr = read_1_bytes(ptr, end_ptr, flag);
   if (ptr == nullptr) return nullptr;
 
   return ptr;
 }
 
-byte *mlog_parse_index(byte *ptr, const byte *end_ptr, dict_index_t **index) {
+const byte *mlog_parse_index(const byte *ptr, const byte *end_ptr,
+                             dict_index_t **index) {
   /* Read the 1 byte for index log version */
   uint8_t index_log_version = 0;
   ptr = parse_index_log_version(ptr, end_ptr, index_log_version);
@@ -1200,10 +1230,9 @@ byte *mlog_parse_index(byte *ptr, const byte *end_ptr, dict_index_t **index) {
     return nullptr;
   }
 
-  byte *ret = nullptr;
   switch (index_log_version) {
     case INDEX_LOG_VERSION_CURRENT:
-      ret = mlog_parse_index_v1(ptr, end_ptr, index);
+      ptr = mlog_parse_index_v1(ptr, end_ptr, index);
       break;
     case INDEX_LOG_VERSION_0:
       /* INDEX_LOG_VERSION_0 is used in 8.0.29 and in 8.0.30 REDO log format
@@ -1216,11 +1245,11 @@ byte *mlog_parse_index(byte *ptr, const byte *end_ptr, dict_index_t **index) {
                 (unsigned int)INDEX_LOG_VERSION_MAX);
   }
 
-  return ret;
+  return ptr;
 }
 
-static byte *mlog_parse_index_v1(byte *ptr, const byte *end_ptr,
-                                 dict_index_t **index) {
+static const byte *mlog_parse_index_v1(const byte *ptr, const byte *end_ptr,
+                                       dict_index_t **index) {
   /* Read the 1 byte flag */
   uint8_t flag = 0;
   ptr = parse_index_flag(ptr, end_ptr, flag);
