@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,17 +28,15 @@
 
 #define JAM_FILE_ID 38
 
-
 struct LqhTransReq {
-  STATIC_CONST( SignalLength = 4 );
-  STATIC_CONST( OldSignalLength = 3 );
+  static constexpr Uint32 SignalLength = 4;
+  static constexpr Uint32 OldSignalLength = 3;
   Uint32 senderData;
   Uint32 senderRef;
   Uint32 failedNodeId;
   Uint32 instanceId;
 };
 
-
 #undef JAM_FILE_ID
 
 #endif
