@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +26,7 @@
 #ifndef ndb_numa_h
 #define ndb_numa_h
 
-#ifdef	__cplusplus
+#ifdef __cplusplus
 extern "C" {
 #endif
 
@@ -37,17 +38,7 @@ extern "C" {
  */
 int NdbNuma_setInterleaved();
 
-#if TODO
-/**
- * Set memory policy that memory allocation should be performed
- *   using interleaving over numa-nodes corresponding to cpu's
- *
- * NOTE: Not thread safe
- */
-int NdbNuma_setInterleavedOnCpus(unsigned cpu[], unsigned len);
-#endif
-
-#ifdef	__cplusplus
+#ifdef __cplusplus
 }
 #endif
 
