@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -227,7 +228,21 @@ class SimpleProperties {
     virtual ~Writer() {}
     virtual bool reset() = 0;
     virtual bool putWord(Uint32 val) = 0;
-    virtual bool putWords(const Uint32 *src, Uint32 len) = 0;
+
+    /**
+     * @brief Put words from char array to the SimpleProperties buffer.
+     *        Function assumes that caller has zero initialized any trailing
+     *        bytes in the char array as it will put every word verbatim.
+     *
+     *        For example, to put 6 characters the char array should look like:
+     *           [M, a, g, n, u, s, 0, 0]
+     *
+     * @param src The array of chars to put words from.
+     * @param len The number of words to put.
+     *
+     * @return false on success
+     */
+    virtual bool putWordsFromChar(const char *src, Uint32 len) = 0;
 
    private:
     bool add(const char *value, int len);
@@ -268,7 +283,7 @@ class LinearWriter : public SimpleProperties::Writer {
 
   bool reset() override;
   bool putWord(Uint32 val) override;
-  bool putWords(const Uint32 *src, Uint32 len) override;
+  bool putWordsFromChar(const char *src, Uint32 len) override;
   Uint32 getWordsUsed() const;
 
  private:
@@ -287,7 +302,7 @@ class UtilBufferWriter : public SimpleProperties::Writer {
 
   bool reset() override;
   bool putWord(Uint32 val) override;
-  bool putWords(const Uint32 *src, Uint32 len) override;
+  bool putWordsFromChar(const char *src, Uint32 len) override;
   Uint32 getWordsUsed() const;
 
  private:
@@ -337,7 +352,7 @@ class SimplePropertiesSectionWriter : public SimpleProperties::Writer {
 
   bool reset() override;
   bool putWord(Uint32 val) override;
-  bool putWords(const Uint32 *src, Uint32 len) override;
+  bool putWordsFromChar(const char *src, Uint32 len) override;
   Uint32 getWordsUsed() const;
 
   /**
