@@ -28,9 +28,9 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <m_ctype.h>
 #include <my_sys.h>
 #include <mysql.h>
+#include "mysql/strings/m_ctype.h"
 
 #include <NdbApi.hpp>
 
