@@ -61,7 +61,7 @@ NdbTransaction *Ndb::doConnect(Uint32 tConNode, Uint32 instance) {
       //****************************************************************************
       DBUG_RETURN(getConnectedNdbTransaction(tConNode, instance));
     } else if (TretCode < 0) {
-      DBUG_RETURN(NULL);
+      DBUG_RETURN(nullptr);
     } else if (TretCode != 0) {
       tAnyAlive = 1;
     }  // if
@@ -83,7 +83,7 @@ NdbTransaction *Ndb::doConnect(Uint32 tConNode, Uint32 instance) {
         //****************************************************************************
         DBUG_RETURN(getConnectedNdbTransaction(tNode, anyInstance));
       } else if (TretCode < 0) {
-        DBUG_RETURN(NULL);
+        DBUG_RETURN(nullptr);
       } else if (TretCode != 0) {
         tAnyAlive = 1;
       }  // if
@@ -109,7 +109,7 @@ NdbTransaction *Ndb::doConnect(Uint32 tConNode, Uint32 instance) {
         //****************************************************************************
         DBUG_RETURN(getConnectedNdbTransaction(tNode, anyInstance));
       } else if (TretCode < 0) {
-        DBUG_RETURN(NULL);
+        DBUG_RETURN(nullptr);
       } else if (TretCode != 0) {
         tAnyAlive = 1;
       }  // if
@@ -132,7 +132,7 @@ NdbTransaction *Ndb::doConnect(Uint32 tConNode, Uint32 instance) {
       theError.code = 4035;
     }
   }  // if
-  DBUG_RETURN(NULL);
+  DBUG_RETURN(nullptr);
 }
 
 int Ndb::NDB_connect(Uint32 tNode, Uint32 instance) {
@@ -740,7 +740,7 @@ NdbTransaction *Ndb::startTransaction(const NdbDictionary::Table *table,
                         (long)(trans ? trans->getTransactionId() : 0)));
     DBUG_RETURN(trans);
   }
-  DBUG_RETURN(NULL);
+  DBUG_RETURN(nullptr);
 }
 
 NdbTransaction *Ndb::startTransaction(Uint32 nodeId, Uint32 instanceId) {
@@ -757,7 +757,7 @@ NdbTransaction *Ndb::startTransaction(Uint32 nodeId, Uint32 instanceId) {
                         (long)(trans ? trans->getTransactionId() : 0)));
     DBUG_RETURN(trans);
   }
-  DBUG_RETURN(NULL);
+  DBUG_RETURN(nullptr);
 }
 
 NdbTransaction *Ndb::startTransaction(const NdbDictionary::Table *table,
@@ -809,7 +809,7 @@ NdbTransaction *Ndb::startTransaction(const NdbDictionary::Table *table,
       DBUG_RETURN(trans);
     }
   } else {
-    DBUG_RETURN(NULL);
+    DBUG_RETURN(nullptr);
   }  // if
 }  // Ndb::startTransaction()
 
@@ -838,15 +838,15 @@ NdbTransaction *Ndb::hupp(NdbTransaction *pBuddyTrans) {
     Uint32 nodeId = pBuddyTrans->getConnectedNodeId();
     NdbTransaction *pCon = startTransactionLocal(
         aPriority, nodeId, refToInstance(pBuddyTrans->m_tcRef));
-    if (pCon == nullptr) DBUG_RETURN(NULL);
+    if (pCon == nullptr) DBUG_RETURN(nullptr);
 
     if (pCon->getConnectedNodeId() != nodeId) {
       // We could not get a connection to the desired node
-      // release the connection and return NULL
+      // release the connection and return nullptr
       closeTransaction(pCon);
       theImpl->decClientStat(TransStartCount, 1); /* Correct stats */
       theError.code = 4006;
-      DBUG_RETURN(NULL);
+      DBUG_RETURN(nullptr);
     }
     pCon->setTransactionId(pBuddyTrans->getTransactionId());
     pCon->setBuddyConPtr((Uint32)pBuddyTrans->getTC_ConnectPtr());
@@ -854,7 +854,7 @@ NdbTransaction *Ndb::hupp(NdbTransaction *pBuddyTrans) {
                         (long)(pCon ? pCon->getTransactionId() : 0)));
     DBUG_RETURN(pCon);
   } else {
-    DBUG_RETURN(NULL);
+    DBUG_RETURN(nullptr);
   }  // if
 }  // Ndb::hupp()
 
@@ -877,27 +877,27 @@ NdbTransaction *Ndb::startTransactionLocal(Uint32 aPriority, Uint32 nodeId,
   DBUG_EXECUTE_IF("ndb_start_transaction_fail", {
     /* Cluster failure */
     theError.code = 4009;
-    DBUG_RETURN(0);
+    DBUG_RETURN(nullptr);
   };);
 #endif
 
   if (unlikely(theRemainingStartTransactions == 0)) {
     theError.code = 4006;
-    DBUG_RETURN(0);
+    DBUG_RETURN(nullptr);
   }
 
   NdbTransaction *tConnection;
   Uint64 tFirstTransId = theFirstTransId;
   tConnection = doConnect(nodeId, instance);
   if (tConnection == nullptr) {
-    DBUG_RETURN(NULL);
+    DBUG_RETURN(nullptr);
   }  // if
 
   theRemainingStartTransactions--;
   NdbTransaction *tConNext = theTransactionList;
   if (tConnection->init()) {
     theError.code = tConnection->theError.code;
-    DBUG_RETURN(NULL);
+    DBUG_RETURN(nullptr);
   }
   theTransactionList = tConnection;  // into a transaction list.
   tConnection->next(tConNext);       // Add the active connection object
@@ -964,7 +964,7 @@ void Ndb::closeTransaction(NdbTransaction *aConnection) {
 
   if (aConnection == nullptr) {
 //-----------------------------------------------------
-// closeTransaction called on NULL pointer, destructive
+// closeTransaction called on nullptr pointer, destructive
 // application behaviour.
 //-----------------------------------------------------
 #ifdef VM_TRACE
@@ -2118,7 +2118,7 @@ const char *Ndb::getNdbErrorDetail(const NdbError &err, char *buff,
   /* If err has non-null details member, prepare a string containing
    * those details
    */
-  if (!buff) DBUG_RETURN(NULL);
+  if (!buff) DBUG_RETURN(nullptr);
 
   if (err.details != nullptr) {
     DBUG_PRINT("info", ("err.code is %u", err.code));
@@ -2145,7 +2145,7 @@ const char *Ndb::getNdbErrorDetail(const NdbError &err, char *buff,
               false);  // FullyQualified names
           if (rc) {
             DBUG_PRINT("info", ("listObjects call 1 failed with rc %u", rc));
-            DBUG_RETURN(NULL);
+            DBUG_RETURN(nullptr);
           }
 
           DBUG_PRINT("info",
@@ -2183,7 +2183,7 @@ const char *Ndb::getNdbErrorDetail(const NdbError &err, char *buff,
 
           if (rc) {
             DBUG_PRINT("info", ("listObjects call 2 failed with rc %u", rc));
-            DBUG_RETURN(NULL);
+            DBUG_RETURN(nullptr);
           }
 
           DBUG_PRINT("info",
@@ -2223,13 +2223,13 @@ const char *Ndb::getNdbErrorDetail(const NdbError &err, char *buff,
            * Strange - perhaps it's been dropped?
            */
           DBUG_PRINT("info", ("Table id %u not found", primTableObjectId));
-          DBUG_RETURN(NULL);
+          DBUG_RETURN(nullptr);
         } else {
           /* Index not found from id - strange.
            * Perhaps it has been dropped?
            */
           DBUG_PRINT("info", ("Index id %u not found", indexObjectId));
-          DBUG_RETURN(NULL);
+          DBUG_RETURN(nullptr);
         }
       }
       case 255:   /* ZFK_NO_PARENT_ROW_EXISTS - Insert/Update failure */
@@ -2249,7 +2249,7 @@ const char *Ndb::getNdbErrorDetail(const NdbError &err, char *buff,
                                             true);  // FullyQualified names
         if (rc) {
           DBUG_PRINT("info", ("listObjects call 1 failed with rc %u", rc));
-          DBUG_RETURN(NULL);
+          DBUG_RETURN(nullptr);
         }
 
         DBUG_PRINT("info", ("Retrieved details for %u foreign keys",
@@ -2270,7 +2270,7 @@ const char *Ndb::getNdbErrorDetail(const NdbError &err, char *buff,
         }
 
         DBUG_PRINT("info", ("Foreign key id %u not found", foreignKeyId));
-        DBUG_RETURN(NULL);
+        DBUG_RETURN(nullptr);
       }
       default: {
         /* Unhandled details type */
@@ -2279,7 +2279,7 @@ const char *Ndb::getNdbErrorDetail(const NdbError &err, char *buff,
   }
 
   DBUG_PRINT("info", ("No details string for this error"));
-  DBUG_RETURN(NULL);
+  DBUG_RETURN(nullptr);
 }
 
 void Ndb::setCustomData(void *_customDataPtr) {
