@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2009, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2009, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -48,4 +48,9 @@ void NDBT_find_ndb_mgmd(BaseString &path);
 */
 void NDBT_find_ndbd(BaseString &path);
 
+/*
+  Wrapper to find ndb_sign_keys in a test environment
+*/
+void NDBT_find_sign_keys(BaseString &path);
+
 #endif
