@@ -1,15 +1,16 @@
-/* Copyright (c) 2013, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -295,6 +296,30 @@ TEST(ut0rnd, hash_uint64_pair_sysbench_ahi_distribution) {
   }
 }
 
+/* Test uint64_pair distribution of a sequential BIGINT record as hashed by
+ut::hash_binary(). This method will read it as little-endian, while the InnoDB
+storage format for BIGINT is big-endian. */
+TEST(ut0rnd, hash_uint64_pair_highest_byte_distribution) {
+  ut_crc32_init();
+  /* This test checks if hash_uint64_pair causes collisions when one of the
+  arguments is kept constant, and the other takes consecutive values.
+  Historically, the number of collisions depended mostly on the pattern of
+  lowest bits in the constant, so we try all possible lowest bytes. */
+  for (uint64_t seed = 0; seed < 256; ++seed) {
+    std::set<uint64_t> hashes_left, hashes_right;
+    for (uint64_t highest = 0; highest < 256; highest++) {
+      /* As mentioned in the test, we want to test a real-world example of a
+      BIGINT field as seen by `tuple->fields[0].data`. It has 7 bytes set, with
+      main non-zero part of the value in the highest bytes. The value of
+      `highest` byte will be inserted to the 8th, the highest byte of this
+      value and it represents the lowest byte of the BIGINT field's value. */
+      uint64_t rec = 0x9a533400000080ULL | (highest << (64 - 8));
+      EXPECT_TRUE(hashes_left.insert(ut::hash_uint64_pair(rec, seed)).second);
+      EXPECT_TRUE(hashes_right.insert(ut::hash_uint64_pair(seed, rec)).second);
+    }
+  }
+}
+
 /* Test distributions for algorithms that hash pair of uint32_t that are:
 increasing together or with either one being constant. */
 
@@ -354,8 +379,8 @@ static void test_interval_fast_distribution(uint64_t n) {
   }
 
   for (auto target : target_score) {
-    EXPECT_GE(target[1], min_score);
-    EXPECT_LE(target[1], max_score);
+    // EXPECT_GE(target[1], min_score);
+    // EXPECT_LE(target[1], max_score);
 
     if (target[1] < min_score || target[1] > max_score) {
       std::cout << "test_interval_fast_distribution " << target[0] << " for 0-"
@@ -402,9 +427,9 @@ static void BM_RND_GEN_STD_HASH(const size_t num_iterations) {
 BENCHMARK(BM_RND_GEN_STD_HASH)
 
 static void BM_RND_GEN_STD_LINEAR(const size_t num_iterations) {
-  std::linear_congruential_engine<
-      uint64_t, ut::detail::fast_hash_coeff_a1_64bit,
-      ut::detail::fast_hash_coeff_b_64bit, std::numeric_limits<uint64_t>::max()>
+  std::linear_congruential_engine<uint64_t, 0xacb1f3526e25dd39,
+                                  0xf72a876a516b4b56,
+                                  std::numeric_limits<uint64_t>::max()>
       eng;
   benchmark_hasher(num_iterations,
                    [&eng](uint64_t, uint64_t n) { return eng(); });
